import 'package:flutter/cupertino.dart';
import 'package:sunny_core_widgets/sunny_core_widgets.dart';
import 'package:sunny_dart/sunny_dart.dart';

Widget crossFade({
  Key? key,
  required first,
  AlignmentGeometry? alignment,
  required second,
  bool showFirst = true,
  Duration? duration,
}) {
  return AnimatedCrossFade(
    key: key,
    firstChild: _renderIfTrue(showFirst == true, first),
    secondChild: _renderIfTrue(showFirst != true, second),
    alignment: alignment = Alignment.centerLeft,
    crossFadeState:
        showFirst ? CrossFadeState.showFirst : CrossFadeState.showSecond,
    duration: duration ?? 300.ms,
    layoutBuilder: defaultLayoutBuilder,
  );
}

Widget _renderIfTrue(bool condition, widgetOrBuilder) {
  if (widgetOrBuilder == null) return emptyBox;
  if (widgetOrBuilder is Widget) return widgetOrBuilder;
  if (condition != true) return emptyBox;
  return Builder(builder: (context) => widgetOrBuilder() as Widget);
}

Widget toggle(first,
    {Key? key,
    AlignmentGeometry? alignment,
    bool isSliver = false,
    bool isShown = true}) {
  return AnimatedCrossFade(
    key: key,
    firstChild: Builder(
      builder: (context) {
        if (isShown == true) {
          return (first is Widget ? first : first(context)) as Widget;
        } else {
          return isSliver ? sliverEmptyBox : emptyBox;
        }
      },
    ),
    secondChild: emptyBox,
    alignment: alignment = Alignment.centerLeft,
    crossFadeState:
        isShown ? CrossFadeState.showFirst : CrossFadeState.showSecond,
    duration: 300.ms,
    layoutBuilder: defaultLayoutBuilder,
  );
}

Widget toggleStream(Widget first,
    {Key? key,
    AlignmentGeometry? alignment,
    required ValueStream<bool> isShown}) {
  return StreamBuilder<bool>(
    key: key,
    initialData: isShown.get()?.resolve(false) ?? false,
    stream: isShown.flatten() as Stream<bool>?,
    builder: (context, snap) => AnimatedCrossFade(
      firstChild: snap.data == true ? first : emptyBox,
      secondChild: emptyBox,
      alignment: alignment = Alignment.centerLeft,
      crossFadeState: (snap.data == true)
          ? CrossFadeState.showFirst
          : CrossFadeState.showSecond,
      duration: 200.milliseconds,
      layoutBuilder: defaultLayoutBuilder,
    ),
  );
}

Widget defaultLayoutBuilder(
    Widget topChild, Key topChildKey, Widget bottomChild, Key bottomChildKey) {
  return Stack(
    alignment: AlignmentDirectional.centerStart,
    clipBehavior: Clip.hardEdge,
    children: <Widget>[
      Positioned(
        key: bottomChildKey,
        left: 0.0,
        top: 0.0,
        right: 0.0,
        child: bottomChild,
      ),
      Positioned(
        key: topChildKey,
        child: topChild,
      ),
    ],
  );
}
