import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:observable_collections/observable_collections.dart';
import 'package:sunny_dart/sunny_dart.dart';
import 'package:sunny_forms/widget_extensions.dart';

typedef HStreamBuilder<T> = Widget Function(BuildContext context, T newElement);
typedef TupleWidgetBuilder<A, B> = Widget Function(
    BuildContext context, A a, B b);

/// Observes a stream.  Get it?? Stream?  Fishing?
class Fisher<T> extends StatelessWidget {
  final Stream<T?> stream;
  final T? initialData;
  final T? fallbackData;
  final HStreamBuilder<T>? builder;
  final String? name;
  final bool isFuture;
  final WidgetBuilder? loadingBuilder;

  Fisher(
      {required ValueStream<T> vstream,
      this.builder,
      this.fallbackData,
      required this.name,
      this.loadingBuilder})
      : stream = vstream.after,
        initialData = vstream.resolve() ?? fallbackData,
        isFuture = false,
        super(key: name.toKey("-fisher"));

  static Fisher<T> of<T>({
    required ValueStream<T> vstream,
    T? fallbackData,
    HStreamBuilder<T>? builder,
    WidgetBuilder? loadingBuilder,
    String? name,
  }) =>
      Fisher<T>(
        name: name,
        vstream: vstream,
        fallbackData: fallbackData,
        builder: builder,
        loadingBuilder: loadingBuilder,
      );

  Fisher.ofKey(
      {required SunnyObservableMap<dynamic, T> source,
      dynamic mapKey,
      T? initialData,
      this.fallbackData,
      this.builder,
      required this.name,
      this.loadingBuilder})
      : stream = source.watchKey(mapKey).after,
        initialData = initialData ?? source[mapKey] ?? fallbackData,
        isFuture = false,
        super(key: name.toKey("-fisher"));

  @override
  Widget build(BuildContext context) {
    return StreamBuilder<T?>(
      key: name.toKey("-fisher-stream"),
      initialData: initialData,
      stream: stream,
      builder: (context, snapshot) {
        if (snapshot.hasData || fallbackData != null) {
          return builder!(context, snapshot.data ?? fallbackData!);
        } else {
          return loadingBuilder?.call(context) ??
              // We were using an empty box, but decided to use a white box to prevent issues where the
              // blank result created a blank overlay
              whiteBox;
        }
      },
    );
  }
}

final whiteBox = Container(color: Colors.white);
