import 'package:flutter/cupertino.dart';
import 'package:sunny_essentials/state.dart';
import 'package:sunny_forms/builders/cross_fade.dart';

class Focused extends StatefulWidget {
  final Widget child;
  final FocusNode focusNode;
  final Widget? unfocused;

  const Focused(
      {Key? key, required this.child, required this.focusNode, this.unfocused})
      : super(key: key);

  @override
  State<StatefulWidget> createState() {
    return _FocusedState();
  }
}

class _FocusedState extends BaseState<Focused> {
  @override
  void initState() {
    super.initState();
    widget.focusNode.addListener(_onFocusChange);
  }

  _onFocusChange() {
    setState(() {});
  }

  @override
  void dispose() {
    widget.focusNode.removeListener(_onFocusChange);
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    if (widget.unfocused != null) {
      return crossFade(
          first: widget.child,
          second: widget.unfocused,
          showFirst: widget.focusNode.hasFocus);
    } else {
      return toggle(widget.child, isShown: widget.focusNode.hasFocus);
    }
  }
}

extension FocusNodeWidget on FocusNode? {
  Widget showWhenFocused(Widget child, {Widget? unfocused, Key? key}) =>
      this == null
          ? child
          : Focused(
              key: key,
              unfocused: unfocused,
              focusNode: this!,
              child: child,
            );
}
