import 'package:flutter/widgets.dart';
import 'package:sunny_core_widgets/sunny_core_widgets.dart';
import 'package:sunny_forms/builders/focused.dart';
import 'package:sunny_forms/raw_text_field.dart';
import 'package:sunny_forms/sunny_form_controller.dart';
import 'package:sunny_forms/sunny_form_field.dart';

/// This widget will clear the contents of a form control (text box)
class ClearInputIcon<T> extends StatelessWidget {
  final TextEditingController? textEditingController;
  final FormValueChanged<T?>? notifier;

  final Widget icon;
  final Widget? expandIcon;
  final FocusNode? focusNode;

  final Widget? unfocused;
  final bool showExpand;

  ClearInputIcon({
    Key? key,
    this.notifier,
    required Widget icon,
    this.textEditingController,
    required this.focusNode,
    this.unfocused,
    required this.showExpand,
    this.expandIcon,
  })  : icon = icon.reIcon(color: sunnyColors.textLight, size: 20),
        super(key: key);

  @override
  Widget build(BuildContext context) {
    return focusNode.showWhenFocused(
      tappable(icon, context: context, onTap: () {
        if (textEditingController?.text == "") {
          focusNode?.unfocus();
        } else {
          notifier?.call(null, AttributeSource.control);
          textEditingController?.text = "";
        }
      }),
      unfocused: unfocused ??
          ((showExpand == true)
              ? tappable(expandIcon!, onTap: () {
                  FocusScope.of(context).requestFocus(focusNode);
                })
              : null),
    );
  }
}
