import 'package:flutter/cupertino.dart';
import 'package:flutter/widgets.dart';

void colorExtensions() {}

extension CupertinoDynamicColorExt on CupertinoDynamicColor? {
  CupertinoDynamicColor? call(BuildContext? context) {
    if (this == null) return null;
    if (context == null) return this;
    return this!.resolveFrom(context);
  }

  /// Forces color resolution to a specified brightness
  Color? force(Brightness? brightness) {
    if (this == null) return null;
    if (brightness == null) return this;
    final resolved =
        brightness == Brightness.dark ? this?.darkColor : this?.color;
    return resolved ?? this!.color;
  }
}

extension DynamicColorExt on Color? {
  Color? resolved(BuildContext context) {
    final self = this;
    if (self == null) return null;
    if (self is CupertinoDynamicColor) return self.resolveFrom(context);

    return self;
  }

  Color? get inverted {
    final self = this;
    if (self is CupertinoDynamicColor) {
      return CupertinoDynamicColor(
        color: self.darkColor,
        darkColor: self.color,
        darkElevatedColor: self.elevatedColor,
        darkHighContrastColor: self.highContrastColor,
        darkHighContrastElevatedColor: self.highContrastElevatedColor,
        elevatedColor: self.darkElevatedColor,
        highContrastColor: self.darkHighContrastColor,
        highContrastElevatedColor: self.darkHighContrastElevatedColor,
      );
    } else {
      return self;
    }
  }
}
