import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter/widgets.dart';
import 'package:flutter_platform_widgets/flutter_platform_widgets.dart';
import 'package:sunny_core_widgets/sunny_core_widgets.dart';

PlatformDialogAction dialogCancelButton(BuildContext context) =>
    PlatformDialogAction(
        onPressed: () => Navigator.pop(context, null),
        child: PlatformText("Cancel"));

Future<bool?> showConfirmDialog(
  BuildContext context, {
  String? title,
  String? subtitle,
  String? confirmButtonLabel,
  String? denyButtonLabel,
  bool isDestructiveAction = true,
}) {
  final intl = SunnyIntl.of(context);
  return showPlatformDialog<bool>(
    context: context,
    barrierDismissible: true,
    builder: (context) {
      return PlatformAlertDialog(
        title: Text(title ?? intl!.areYouSure),
        content: subtitle == null
            ? null
            : Padding(
                padding: const EdgeInsets.only(top: 8.0),
                child: Text(
                  subtitle,
                  style: TextStyle(height: 16 / 14),
                ),
              ),
        actions: [
          PlatformDialogAction(
            key: Key("confirm-no"),
            child: Text(denyButtonLabel ?? intl!.no),
            onPressed: () {
              Navigator.of(context).pop(false);
            },
          ).build(context),
          PlatformDialogAction(
            key: Key("confirm-yes"),
            child: Text(confirmButtonLabel ?? intl!.yes),
            cupertino: (context, _) => CupertinoDialogActionData(
                isDefaultAction: true,
                isDestructiveAction: isDestructiveAction),
            onPressed: () async {
              Navigator.of(context).pop(true);
            },
          ).build(context)
        ],
      );
    },
  );
}

Future<bool?> showAlertDialog(
  BuildContext context, {
  String? title,
  String? subtitle,
  String? confirmButtonLabel,
}) {
  final intl = SunnyIntl.of(context);
  return showPlatformDialog<bool>(
    context: context,
    barrierDismissible: true,
    builder: (context) {
      return PlatformAlertDialog(
        title: Text(title ?? intl!.areYouSure),
        content: subtitle == null
            ? null
            : Padding(
                padding: const EdgeInsets.only(top: 8.0),
                child: Text(
                  subtitle,
                  style: TextStyle(height: 16 / 14),
                ),
              ),
        actions: [
          PlatformDialogAction(
            key: Key("confirm-yes"),
            child: Text(confirmButtonLabel ?? "OK"),
            cupertino: (context, _) => CupertinoDialogActionData(
                isDefaultAction: true, isDestructiveAction: false),
            onPressed: () async {
              Navigator.of(context).pop(true);
            },
          ).build(context)
        ],
      );
    },
  );
}

Future<bool> showConfirmDeleteDialog(BuildContext context,
    {String? label, String? fullLabel}) async {
  final didConfirm = await showPlatformDialog<bool>(
      context: context,
      builder: (context) {
        var intl = SunnyIntl.of(context)!;
        return PlatformAlertDialog(
          title: Text(fullLabel ?? intl.deleteConfirmTitle(label!)),
          material: (context, platform) => MaterialAlertDialogData(
            buttonPadding: const EdgeInsets.all(8),
            actionsPadding: const EdgeInsets.all(8),
          ),
          actions: [
            PlatformDialogAction(
              child: Text(intl.deleteLabel),
              cupertino: (context, _) =>
                  CupertinoDialogActionData(isDestructiveAction: true),
              onPressed: () async => Navigator.of(context).pop(true),
            ),
            PlatformDialogAction(
              child: Text(intl.cancelLabel),
              cupertino: (context, _) =>
                  CupertinoDialogActionData(isDefaultAction: true),
              onPressed: () async => Navigator.of(context).pop(false),
            ),
          ],
        );
      });
  return didConfirm == true;
}
