import 'dart:async';

import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter/widgets.dart';
import 'package:google_maps_webservice/places.dart' as gmaps;
import 'package:logging/logging.dart';
import 'package:sunny_core_widgets/sunny_core_widgets.dart';
import 'package:sunny_forms/form_fields/location/address_picker.dart';
import 'package:sunny_forms/sunny_fake_field_container.dart';
import 'package:sunny_forms/sunny_form_controller.dart';
import 'package:sunny_forms/sunny_form_field.dart';
import 'package:sunny_forms/widget_extensions.dart';
import 'package:sunny_sdk_core/sunny_sdk_core.dart';
import 'package:sunny_service_stubs/models.dart';

import 'location_tile.dart';
import 'location_utils.dart';

final _log = Logger("locationPicker");

Future<SunnyPlace?> openLocationPicker(BuildContext context,
    {SunnyPlace? toUpdate, Set<String>? locationTypes}) async {
  try {
    final place =
        await openAddressPicker(context, locationTypes: locationTypes);

    if (place == null) return null;
    final SunnyPlace location = toUpdate ?? IPhysicalLocation.of();
    final placesApi = gmaps.GoogleMapsPlaces(
        apiKey: ThirdPartyCreds.googlePlaces.credentials);
    final response = await placesApi.getDetailsByPlaceId(place.placeId!);
    final details = response.result;
    location.fromGooglePlaceDetails(details, placeId: place.placeId);
    location.prune(PhysicalLocationFields.values);
    return location;
  } catch (e, stack) {
    print(e);
    _log.severe(e, null, stack);
    return null;
  }
}

class LocationPickerControl extends StatefulWidget {
  final FormValueChanged<IPhysicalLocation> onChange;
  final IPhysicalLocation initialValue;
  final bool? enabled;
  final Set<String>? locationTypes;

  const LocationPickerControl({
    Key? key,
    required this.onChange,
    required this.initialValue,
    this.enabled,
    this.locationTypes,
  }) : super(key: key);

  @override
  _LocationPickerControlState createState() => _LocationPickerControlState();
}

class _LocationPickerControlState extends State<LocationPickerControl> {
  IPhysicalLocation? _location;

  @override
  void initState() {
    super.initState();
    _location = widget.initialValue;
  }

  @override
  Widget build(BuildContext context) {
    return Container(
      child: GestureDetector(
        onTap: () async {
          print("Tapped the edit");
          final location = await openLocationPicker(context,
              locationTypes: widget.locationTypes);
          print("Got an address back: $location");
          if (location != null) {
            widget.onChange
                .call(location as IPhysicalLocation, AttributeSource.control);

            setState(() {
              print("Setting this address here: $location");
              this._location = location;
            });
          }
        },
        child: FakeFieldContainer(
          placeholder: "Click to set a location",
          icon: context.icons.location,
          focused: false,
          value: LocationTile(
            showOpenInMapsLink: widget.enabled == false,
            emptyState: PlaceholderText("Click to set a location"),
            place: _location,
          ),
        ),
      ),
    );
  }
}
