import 'dart:async';

import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter/widgets.dart';
import 'package:flutter_platform_widgets/flutter_platform_widgets.dart';
import 'package:flutter_typeahead/flutter_typeahead.dart';
import 'package:sunny_core_widgets/sunny_core_widgets.dart';
import 'package:sunny_dart/json.dart';
import 'package:sunny_dart/sunny_dart.dart';
import 'package:sunny_dart/typedefs.dart';
import 'package:sunny_essentials/state.dart';
import 'package:sunny_forms/attribute_form_field.dart';
import 'package:sunny_forms/clear_input.dart';
import 'package:sunny_forms/form_fields/typeaheads.dart';
import 'package:sunny_forms/icon/image_widgets.dart';
import 'package:sunny_forms/options/options_service.dart';
import 'package:sunny_forms/options/options_view_service.dart';
import 'package:sunny_forms/raw_text_field.dart';
import 'package:sunny_forms/sunny_colors.dart';
import 'package:sunny_forms/sunny_form_controller.dart';
import 'package:sunny_forms/sunny_form_field.dart';
import 'package:sunny_forms/widget_extensions.dart';
import 'package:sunny_sdk_core/sunny_sdk_core.dart';
import 'package:sunny_service_stubs/options.dart';

typedef ExpectedValue<K, T> = dynamic Function(KeyedOption<K, T>? selected);
const kInputIconSize = 20.0;

class OptionSelectorField<K, T> extends StatefulWidget {
  final JsonPath attribute;
  final String label;
  final String? placeholder;
  final String? noOptionsLabel;
  final KeyedOptionsHandler<K, T> handler;
  final TypeaheadHandler<K, T> viewHandler;
  final Mapping<T, dynamic> converter;
  final TypeaheadFocusMode? focusMode;
  final bool? autofocus;
  final bool isRequired;
  final bool? enabled;
  final ExpectedValue<K, T> expected;
  final bool _isKey;

  OptionSelectorField.ofValue({
    Key? key,
    required JsonPath attribute,
    required String label,
    String? noOptionsLabel,
    bool? autofocus,
    bool? enabled,
    String? placeholder,
    required bool isRequired,
    required KeyedOptionsHandler<K, T> handler,
    required TypeaheadHandler<K, T> viewHandler,
    TypeaheadFocusMode? focusMode,
  }) : this._(
          false,
          expected: ((option) => option?.value),
          key: key,
          attribute: attribute,
          label: label,
          enabled: enabled,
          noOptionsLabel: noOptionsLabel,
          autofocus: autofocus,
          placeholder: placeholder,
          isRequired: isRequired,
          handler: handler,
          viewHandler: viewHandler,
          focusMode: focusMode,
        );

  OptionSelectorField.ofKey({
    Key? key,
    required JsonPath attribute,
    required String label,
    String? noOptionsLabel,
    bool? autofocus,
    bool? enabled,
    String? placeholder,
    required bool isRequired,
    required KeyedOptionsHandler<K, T> handler,
    required TypeaheadHandler<K, T> viewHandler,
    TypeaheadFocusMode? focusMode,
  }) : this._(
          true,
          expected: ((option) => option?.key),
          key: key,
          attribute: attribute,
          label: label,
          enabled: enabled,
          noOptionsLabel: noOptionsLabel,
          autofocus: autofocus,
          placeholder: placeholder,
          isRequired: isRequired,
          handler: handler,
          viewHandler: viewHandler,
          focusMode: focusMode,
        );

  OptionSelectorField._(
    this._isKey, {
    required this.expected,
    Key? key,
    required this.attribute,
    this.enabled,
    required this.label,
    String? noOptionsLabel,
    this.autofocus,
    String? placeholder,
    required this.isRequired,
    required this.handler,
    required this.viewHandler,
    Mapping<T, dynamic>? converter,
    TypeaheadFocusMode? focusMode,
  })  : converter = converter ?? ((input) => input),
        focusMode = focusMode ?? viewHandler.options?.focusMode,
        placeholder = placeholder ?? viewHandler.options?.placeholder,
        noOptionsLabel = noOptionsLabel ?? viewHandler.options?.noOptionsLabel,
        super(key: key);

  static OptionSelectorField<String, String> ofString({
    Key? key,
    required JsonPath attribute,
    required String label,
    String? noOptionsLabel,
    String? placeholder,
    bool? autofocus,
    required bool isRequired,
    required OptionsHandler<String> handler,
    required TypeaheadHandler<String, String> viewHandler,
    TypeaheadFocusMode? focusMode,
  }) {
    return OptionSelectorField<String, String>.ofKey(
      key: key,
      attribute: attribute,
      label: label,
      isRequired: isRequired,
      noOptionsLabel: noOptionsLabel,
      autofocus: autofocus,
      placeholder: placeholder,
      handler: handler,
      viewHandler: viewHandler,
      focusMode: focusMode,
    );
  }

  @override
  State<StatefulWidget> createState() {
    return _isKey
        ? _OptionSelectorFieldState<K, T>.ofKey()
        : _OptionSelectorFieldState<K, T>.ofValue();
  }
}

class _OptionSelectorFieldState<K, T>
    extends BaseState<OptionSelectorField<K, T>> {
  JsonPath get path => widget.attribute;

  /// Determines whether this option selector is ultimately storing an embedded [V] or the key [K]
  final bool isKey;

  _OptionSelectorFieldState.ofKey() : isKey = true;
  _OptionSelectorFieldState.ofValue() : isKey = false;

  @override
  Widget build(BuildContext context) {
    final keyGetter = (SunnyFormFieldState<dynamic> state) {
      final curr = state.value;
      if (curr == null) {
        return null;
      }
      if (curr is K) {
        return curr;
      } else if (curr is T) {
        return widget.handler.toKey(curr);
      } else {
        return illegalState<K>("Invalid state");
      }
    };
    return AttributeFormField<Object>.forInitialValueGetter(
      attribute: widget.attribute,
      label: widget.label,
      isRequired: widget.isRequired,
      placeholder: widget.placeholder,
      builder: (context, state) {
        return OptionSelectorControl<K, T>(
          key: Key("field-$path"),
          handler: widget.handler,
          viewHandler: widget.viewHandler,
          focusNode: state.focusNode,
          focusMode: widget.focusMode,
          enabled: widget.enabled,
          isRequired: widget.isRequired == true,
          noOptionsLabel: widget.noOptionsLabel,
          autofocus: widget.autofocus ?? false,
          onChange: (KeyedOption<K, T>? value, source) {
            state.updateValue(widget.expected(value), source);
          },
          onError: (error) => state.error = error,
          initialKeyValue: keyGetter(state),
//          viewHandler: widget.viewHandler,
          placeholder: widget.placeholder,
        );
      },
    );
  }
}

class OptionSelectorControl<K, T> extends StatefulWidget {
//  final String selectedType;
  final FormValueChanged<KeyedOption<K, T>?> onChange;
  final ValueChanged<ValidationError> onError;
  final K? initialKeyValue;
  final TypeaheadHandler<K, T> viewHandler;
  final T? initialValue;
  final KeyedOptionsHandler<K, T> handler;
  final String? placeholder;
  final bool? enabled;
  final String? noOptionsLabel;
  final FocusNode? focusNode;
  final bool autofocus;
  final Widget? prefixIcon;
  final bool showExpand;
  final TypeaheadFocusMode? focusMode;
  final TextEditingController? textController;
  final bool? isRequired;
  final ExpectedValue<K, T> expected;
  final Predicate? resultFilter;

  OptionSelectorControl({
    Key? key,
    required this.handler,
    required this.onChange,
    required this.onError,
    required this.viewHandler,
    Widget? prefixIcon,
    ExpectedValue<K, T>? expected,
    this.initialValue,
    bool? showExpand,
    this.enabled,
    this.resultFilter,
    this.initialKeyValue,
    String? placeholder,
    String? noOptionsLabel,
    this.focusNode,
    TypeaheadFocusMode? focusMode,
    required this.isRequired,
    this.autofocus = false,
    this.textController,
  })  : placeholder = placeholder ?? viewHandler.options?.placeholder,
        expected = expected ?? ((input) => input),
        noOptionsLabel = noOptionsLabel ?? viewHandler.options?.noOptionsLabel,
        focusMode = focusMode ?? viewHandler.options?.focusMode,
        showExpand = showExpand ??
            viewHandler.options?.showExpand ??
            (focusMode ?? viewHandler.options?.focusMode).isShowAll,
        prefixIcon = prefixIcon ?? viewHandler.options?.prefixIcon as Widget?,
        super(key: key);

  @override
  _OptionSelectorControlState<K, T> createState() =>
      _OptionSelectorControlState<K, T>();
}

class _OptionSelectorControlState<K, T>
    extends BaseState<OptionSelectorControl<K, T>> {
  TextEditingController? _textEditingController;
  KeyedOption<K, T>? _selectedItem;
  bool? shouldFilter;
  FocusNode? _focusNode;

  String? uid;
  @override
  void initState() {
    super.initState();
    uid = uuid();
    FutureOr<KeyedOption<K, T>?> initialOption;
    if (widget.initialKeyValue != null) {
      initialOption = handler.resolveOption(widget.initialKeyValue);
    } else if (widget.initialValue != null) {
      initialOption = handler.toOption(widget.initialValue);
    }
    _focusNode =
        widget.focusNode ?? FocusNode(debugLabel: "Option selector control");
    _focusNode!.addListener(_onFocusChange);
    final option = initialOption.resolveOrNull();
    _textEditingController = widget.textController ??
        TextEditingController.fromValue(
          textEditingValue(viewHandler.getSelection(option!)),
        );
    _textEditingController!.addListener(_onInputChange);

    if (initialOption is Future<KeyedOption<K, T>>) {
      initialOption.futureValue().then(_updateSelectedOption);
    } else {
      _updateSelectedOption(initialOption as KeyedOption<K, T>?,
          applyState: false);
    }
  }

  void _updateSelectedOption(KeyedOption<K, T>? value,
      {bool applyState = true}) {
    this._selectedItem = value;
    if (applyState) setState(() {});
    _textEditingController!.text = viewHandler.getSelection(value) ?? '';
  }

  @override
  void dispose() {
    if (widget.textController == null) {
      _textEditingController!.dispose();
    } else {
      _textEditingController!.removeListener(_onInputChange);
    }
    _focusNode!.removeListener(_onFocusChange);
    super.dispose();
  }

  void _onInputChange() {
    final hasValue = _textEditingController!.text.isNotNullOrBlank;
    if (hasValue && shouldFilter == null) {
      shouldFilter = false;
    } else if (hasValue && shouldFilter == false) {
      shouldFilter = true;
    }
  }

  void _onFocusChange() {
    if (_focusNode!.hasFocus != true) {
      /// Clean up - if they typed something that wasn't a valid option, clear it out

      _textEditingController!.text =
          viewHandler.getSelection(_selectedItem) ?? "";
      shouldFilter = null;
    } else {
      final focusMode = widget.focusMode ?? TypeaheadFocusMode.showFiltered;
      switch (focusMode) {
        case TypeaheadFocusMode.showAll:
          shouldFilter = null;
          break;
        case TypeaheadFocusMode.showAllAndClear:
          shouldFilter = null;
          _textEditingController!.text = "";
          break;
        case TypeaheadFocusMode.showFiltered:
          break;
        case TypeaheadFocusMode.none:
          break;
      }
    }
  }

  String get query => _textEditingController!.text;

  TextEditingValue textEditingValue(String? text) {
    text ??= "";
    return TextEditingValue(
      text: text,
      selection: TextSelection.collapsed(offset: text.length),
    );
  }

  KeyedOptionsHandler<K, T> get handler => widget.handler;
  TypeaheadHandler<K, T> get viewHandler => widget.viewHandler;

  @override
  Widget build(BuildContext context) {
    return inputWithIcon(
      prefix: imageWidgetOf(
        context,
        _selectedItem?.icon ?? widget.prefixIcon,
        size: kInputIconSize,
        circle: false,
      ),
      suffix: ClearInputIcon(
        key: Key("clearInput-$uid"),
        focusNode: widget.focusNode,
        notifier: widget.onChange,
        textEditingController: _textEditingController,
        showExpand: false,
        icon: Icon(Icons.clear),
      ),
      // suffix: ClearInputIcon(
      //   focusNode: _focusNode,
      //   icon: Icon(Icons.clear),
      //   notifier: widget.onChange,
      //   // We pass [false] in this case because we want to display the chevron, which is the secondary option
      //   unfocused: widget.viewHandler?.renderSelectedItemSuffix(
      //       context, _selectedItem,
      //       selectOption: _onSuggestionSelected),
      //   textEditingController: _textEditingController,
      //   showExpand: widget.showExpand,
      // ),
      color: _focusNode!.hasFocus == true || _selectedItem != null
          ? sunnyColors.primaryColor
          : sunnyColors.textLight,
      input: TypeAheadField<KeyedOption<K, T>?>(
        autoFlipDirection: true,
        suggestionsCallback: (query) async {
          final _filtered = await handler.filter(
              (shouldFilter == true) ? query : null, [], 10);
          var filtered = _filtered.where((element) =>
              widget.resultFilter == null || widget.resultFilter!(element));
          if (filtered.isEmpty &&
              query.isNotNullOrBlank &&
              handler.adhocOptionBuilder != null) {
            final result = handler.adhocOptionBuilder!(query);
            return [if (result != null) result];
          }
          return filtered.map((t) => handler.toOption(t)).toList();
        },
        onSuggestionSelected: _onSuggestionSelected,
        noItemsFoundBuilder: (context) {
          /// If there's no query text, and we don't have any options
          if (query.isNotNullOrBlank) {
            return buildNoItemsTile(context,
                message: widget.noOptionsLabel ?? "Start typing above");
          } else {
            return emptyBox;
          }
        },
        keepSuggestionsOnLoading: true,
        loadingBuilder: (context) {
          return pad(PlatformCircularProgressIndicator());
        },
        itemBuilder: (BuildContext context, KeyedOption<K, T>? item) {
          final isSelected = item?.key == _selectedItem?.key;
          return widget.viewHandler
              .wrapSuggestionTile(widget.viewHandler.renderSuggestionTile(
            context,
            item!,
            isSelected: isSelected,
            selectOption: _onSuggestionSelected,

            /// We pass null for the onTap handler because the parent control [TypeaheadField] takes care of listening
            /// to the taps for us
            onTap: null,
          ));
        },
        textFieldConfiguration: getTypeaheadInputConfig(
          context,
          controller: _textEditingController,
          enabled: widget.enabled != false,
          placeholder: widget.placeholder,
          inputAction: TextInputAction.done,
          focusNode: _focusNode,
          autofocus: widget.autofocus,
          hasPrefix: _selectedItem?.icon != null || widget.prefixIcon != null,
          hasSuffix: true,
        ),
        animationStart: 1.0,
        transitionBuilder: (context, widget, controller) {
          return widget;
        },
        getImmediateSuggestions: widget.focusMode != TypeaheadFocusMode.none,
        hideOnLoading: false,
        hideOnEmpty: false,
        hideOnError: false,
        hideSuggestionsOnKeyboardHide: false,
        errorBuilder: typeaheadErrorBuilder,
      ),
    );
  }

  _onSuggestionSelected(KeyedOption<K, T>? option) async {
    KeyedOption<K, T>? _option = option;
    if (option is KeyedAdhocOption<K, T> &&
        option.hasAdhocCreator &&
        option.label.isNotNullOrBlank) {
      final result = await option.runAdhocCreator(context);
      if (result == null) {
        return;
      }
      _option = handler.toOption(result);
      widget.onChange.call(_option, AttributeSource.control);
      _updateSelectedOption(_option!);
//            setState(() {
//              _textEditingController.text = _option.selection;
//              icon = imageWidgetOf(_option.icon, size: 25, circle: false);
//            });
    } else {
      widget.onChange.call(option, AttributeSource.control);
      _updateSelectedOption(_option!);

//            setState(() {
//              _textEditingController.text = option.selection;
//              icon = imageWidgetOf(option.icon, size: 25, circle: false);
//            });
    }
  }
}

class TypeaheadOption extends StatelessWidget {
  final Widget? child;

  const TypeaheadOption({Key? key, this.child}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Container(
      decoration: BoxDecoration(
        border: Border(bottom: SunnyDeco.borderSideLight),
      ),
      child: child,
    );
  }
}
