import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter_typeahead/flutter_typeahead.dart';
import 'package:logging/logging.dart';
import 'package:sunny_forms/dividing_line.dart';
import 'package:sunny_forms/shared/theme_helpers.dart';

final _log = Logger("typeahead");

Widget buildNoItemsTile(BuildContext context, {String? message}) {
  return Container(
    padding: const EdgeInsets.symmetric(vertical: 8.0),
    child: Text(
      message ?? "No Result",
      textAlign: TextAlign.center,
      style: TextStyle(color: Theme.of(context).disabledColor, fontSize: 18.0),
    ),
  );
}

TextFieldConfiguration getTypeaheadInputConfig(
  BuildContext context, {
  String? placeholder,
  TextInputAction? inputAction,
  FocusNode? focusNode,
  TextEditingController? controller,
  InputDecoration? decoration,
  bool? autofocus = false,
  bool enabled = true,
  bool hasPrefix = false,
  bool hasSuffix = false,
}) {
  final baseDecoration =
      decoration ?? InputDecoration.collapsed(hintText: placeholder);
  return TextFieldConfiguration(
      autofocus: autofocus == true,
      autocorrect: false,
      enabled: enabled,
      controller: controller,
      focusNode: focusNode,
      textInputAction: inputAction,
      keyboardAppearance: Brightness.light,
      style: sunnyText.input0,
      keyboardType: TextInputType.text,
      decoration: applyTheme(
        context,
        decoration: baseDecoration,
      ).copyWith(
          contentPadding: inputPadding(
        prefix: hasPrefix,
        suffix: hasSuffix,
      )));
}

Widget typeaheadErrorBuilder(BuildContext context, Object? err) {
  _log.info("SMART DATE: $err");
  if (err is Error) _log.info(err.stackTrace);
  return Padding(
    padding: const EdgeInsets.all(8.0),
    child: Text(
      'Error: $err',
      style: TextStyle(color: Theme.of(context).errorColor),
    ),
  );
}
