import 'package:cached_network_image/cached_network_image.dart';
import 'package:flutter/material.dart';
import 'package:flutter/widgets.dart';
import 'package:flutter_staggered_grid_view/flutter_staggered_grid_view.dart';
import 'package:sunny_dart/sunny_dart.dart';
import 'package:sunny_essentials/sunny_essentials.dart';
import 'package:sunny_sdk_core/api/api_client_getter.dart';
import 'package:sunny_service_stubs/models.dart';

class ImageDisplayGallery extends StatelessWidget {
  final List<IImageContent> imageList;

  const ImageDisplayGallery({Key? key, this.imageList = const []})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    return StaggeredGridView.countBuilder(
      crossAxisCount: 4,
      itemCount: imageList.length,
      controller: PrimaryScrollController.of(context),
      primary: false,
      shrinkWrap: true,
      itemBuilder: (context, idx) {
        try {
          final img = imageList[idx];
          double? height;
          if (img.dimensions?.height != null) {
            height = img.dimensions!.height! * 200.0 / img.dimensions!.width!;
          }
          return CachedNetworkImage(
            imageUrl: img.imageUrl!,
            httpHeaders: {
              "Authorization": "Bearer ${apiClient.currentAccessToken}"
            },
            fit: BoxFit.cover,
            height: height,
            errorWidget: (context, msg, uri) {
              return ErrorText(msg);
            },
          );
        } catch (e, stack) {
          print(e);
          print(stack);
          return sliverBox(ErrorText("$e"));
        }
      },
      staggeredTileBuilder: (int idx) =>
          getStaggeredTileCount(idx, imageList.length),
      //staggeredTileBuilder: (int idx) => StaggeredTile.count(2,idx.isEven ? 2 : 1),
      mainAxisSpacing: 4.0,
      crossAxisSpacing: 4.0,
    );
    /*
    return WaterfallFlow.builder(
      shrinkWrap: true,
      primary: false,
      controller: PrimaryScrollController.of(context),
      itemCount: imageList.length,
      padding: EdgeInsets.zero,
      itemBuilder: (context, idx) {
        try {
          final img = imageList[idx];
          double height;
          if (img.dimensions?.height != null) {
            height = img.dimensions.height * 200.0 / img.dimensions.width;
          }
          return Container(
            color: Colors.red,
            height: 80,
            child: CachedNetworkImage(
              imageUrl: img.imageUrl,
              httpHeaders: {"Authorization": apiClient.authHeader},
              fit: BoxFit.cover,
              height: height,
              errorWidget: (context, msg, uri) {
                return ErrorText(msg);
              },
            ),
          );
        } catch (e, stack) {
          print(e);
          print(stack);
          return sliverBox(ErrorText("$e"));
        }
      },
      gridDelegate: SliverWaterfallFlowDelegateWithMaxCrossAxisExtent(
          maxCrossAxisExtent: 200,
          crossAxisSpacing: 0.0,
          mainAxisSpacing: 0.0,
          lastChildLayoutTypeBuilder: (idx) {
            return idx == imageList.length
                ? LastChildLayoutType.foot
                : LastChildLayoutType.none;
          }),
    );

     */
  }

  /*
    Based on 4,4
   */
  StaggeredTile getStaggeredTileCount(int index, int size) {
    int multiplier = (size / 4).floor();
    /*
      n = multiplier
      4(n)+1, 4(n)+2, 4(n)+3
     */
    if ((size == (4 * multiplier) + 1) && size == (index + 1)) {
      //if ((size == 1 || size == 5 || size == 9 || size == 13) && size == (index + 1)) {
      // 4(n) + 1
      return StaggeredTile.count(4, 2);
    } else if ((size == (4 * multiplier) + 2) && size == (index + 1)) {
      //if ((size == 2 || size == 6 || size == 10 || size == 14) && size == (index + 1)) {
      // 4(n) + 2
      return StaggeredTile.count(2, 2);
    } else if ((size == (4 * multiplier) + 3) && size == (index + 1)) {
      //if ((size == 3 || size == 7 || size == 11 || size == 15) && size == (index + 1)) {
      // 4(n) + 3
      return StaggeredTile.count(2, 1);
    } else {
      // 4(n), alternate odd and even shape
      return StaggeredTile.count(2, index.isEven ? 2 : 1);
    }
  }

  static ImageDisplayGallery? ofList({List<IImageContent>? imageList}) {
    if (imageList.isNullOrEmpty) {
      return null;
    } else {
      return ImageDisplayGallery(imageList: imageList!);
    }
  }
}
