import 'package:flutter/widgets.dart';
import 'package:image_cropper/image_cropper.dart';
import 'package:logging/logging.dart';
import 'package:pfile_picker/pfile_picker.dart';
import 'package:sunny_core_widgets/routes/modals.dart';
import 'package:sunny_fluro/sunny_fluro.dart';
import 'package:sunny_forms/image/crop_controls.dart';
import 'package:sunny_forms/image/cropper.dart';

final _log = Logger("croppers");

typedef CropFn = Future<PlatformFile> Function(
    BuildContext context, PlatformFile file);

class Croppers {
  static Future<PFile?> cropFallback(BuildContext context, PFile source) async {
    try {
      // ignore: deprecated_member_use
      var bytes = source.bytes ?? await source.awaitData;
      // ignore: missing_required_param
      return await Modals.open<PFile>(
        context,
        builder: (context) {
          return CropImagePage.ofPlatformFile(
            source: source,
            bytes: bytes,
          );
        },
        settings: PathRouteSettings(route: "/crop", label: "Crop Image"),
      );
    } catch (e, stack) {
      _log.severe("Error $e", e, stack);
      rethrow;
    }
  }

  static Future<PFile?> cropperFallback(
      BuildContext context, PFile source) async {
    try {
      final bytes = await source.awaitData;
      // ignore: missing_required_param
      return await Modals.open<PFile>(
        context,
        builder: (context) {
          return Cropper(image: bytes);
        },
      );
    } catch (e, stack) {
      _log.severe("Error $e", e, stack);
      rethrow;
    }
  }

  static Future<PFile?> cropNative(
      BuildContext context, PFile imageFile) async {
    final cropped = await ImageCropper.cropImage(
      sourcePath: imageFile.path!,
      iosUiSettings: IOSUiSettings(
        showActivitySheetOnDone: false,
        hidesNavigationBar: true,
      ),
      aspectRatio: CropAspectRatio(ratioX: 1, ratioY: 1),
      maxWidth: 2048,
      maxHeight: 2048,
    );

    if (cropped == null) return null;
    return PFile.of(cropped);
  }
}
