import 'package:flutter/widgets.dart';
import 'package:meta_forms/meta_forms.dart';
import 'package:sunny_sdk_core/sunny_sdk_core.dart';
import 'package:sunny_dart/json/json_path.dart';
import 'package:sunny_forms/meta/meta_form.dart';
import 'package:sunny_forms/meta/meta_form_context.dart';
import 'package:sunny_forms/meta/meta_property_handler.dart';
import 'package:sunny_forms/sunny_form_controller.dart';
import 'package:sunny_forms/sunny_form_field.dart';
import 'package:sunny_service_stubs/options.dart';

/// Renders inline embedded entities.
class CollapsingEmbedPropertyHandler<T> extends MetaPropertyHandler<T> {
  final Set<MSchemaRef> _ignoredTypes;

  CollapsingEmbedPropertyHandler(this._ignoredTypes);

  @override
  List<HandledPaths> acceptProperties(
    JsonPath? basePath,
    IMSchemaDefinition definition,
    Map<JsonPath, IMSchemaProperty> available,
  ) =>
      [
        ...available.values
            .whereType<IEmbeddedProperty>()
            .where((final p) =>
                p.embedRef != null && !_ignoredTypes.contains(p.embedRef))
            .map((p) => HandledPaths.ofProperty(
                  p,
                  basePath!,
                  this,
                  data: {"embedRef": p.embedRef},
                )),
      ];

  @override
  Iterable<Widget> renderProperty(
      BuildContext context,
      IMetaFormContext metaForm,
      HandledPaths paths,
      SunnyFormFieldState<T> state,
      {Key? key}) {
    final MSchemaRef embedRef = paths.data["embedRef"] as MSchemaRef;
    final embedPath = paths.path;
    final form = context.form();
    MModel? subject = context.form().get(embedPath);
    if (subject == null) {
      subject = embedRef.newInstance();
      form.set(embedPath, subject);
    }
    return [
      MetaForm(
        name: "embed-${paths.path.toKey()}",
        metaFormContext: MetaFormContext.embedded(
          context,
          ref: embedRef,
          embedPath: embedPath,
          overrides: metaForm.overrides,
          extraHandlers: metaForm.extraHandlers.cast(),
          scope: metaForm.scope,
          subject: subject!,
        ),
      )
    ];
  }
}
