import 'package:flexidate/flexidate.dart';
import 'package:flutter/widgets.dart';
import 'package:meta_forms/meta_forms.dart';
import 'package:sunny_dart/json/json_path.dart';
import 'package:sunny_dart/sunny_dart.dart';
import 'package:sunny_forms/form_fields/duration_picker.dart';
import 'package:sunny_forms/meta/meta_property_handler.dart';
import 'package:sunny_forms/sunny_form_field.dart';
import 'package:sunny_sdk_core/model/time_unit.dart';
import 'package:sunny_sdk_core/sunny_sdk_core.dart';
import 'package:sunny_service_stubs/options.dart';

class DurationPropertyHandler extends MetaPropertyHandler<dynamic> {
  @override
  List<HandledPaths> acceptProperties(
      JsonPath? basePath,
      IMSchemaDefinition definition,
      Map<JsonPath, IMSchemaProperty> available) {
    IMSchemaProperty? durationUnit;
    IMSchemaProperty? durationAmount;

    available.forEach((path, prop) {
      if (path.last == "durationUnit") durationUnit = prop;
      if (path.last == "durationAmount") durationAmount = prop;
    });

    return [
      if (durationUnit != null && durationAmount != null)
        HandledPaths.ofProperties(
            {durationAmount, durationUnit}, basePath!, this,
            data: {
              DurationProperty.unit: durationUnit,
              DurationProperty.amount: durationAmount,
            }),
    ];
  }

  @override
  Iterable<Widget> renderProperty(
      BuildContext context,
      IMetaFormContext metaForm,
      HandledPaths paths,
      SunnyFormFieldState<dynamic> state,
      {Key? key}) {
    final IMSchemaProperty? amountProp =
        paths.data[DurationProperty.amount] as IMSchemaProperty?;
    final IMSchemaProperty? unitProp =
        paths.data[DurationProperty.unit] as IMSchemaProperty?;
    // ignore: unused_local_variable
    final basePath = paths.basePath;
    final rawValue = state.value;
    TimeSpan? initial;
    if (rawValue is int) {
      initial = TimeSpan(minutes: rawValue);
    } else if (rawValue is TimeSpan) {
      initial = rawValue;
    } else if (rawValue == null) {
      initial = TimeSpan(hours: 1);
    } else {
      initial = illegalState("Invalid type ${initial.runtimeType}");
    }

    return [
      DurationPickerControl(
        key: key,
        onChange: (value) {
          state.form!.set(unitProp!.jsonPath, TimeUnit.minutes_);
          state.form!.set(amountProp!.jsonPath, value.minutes);
          //state.updateValue(value, AttributeSource.control);
        },
//        amountAttribute: basePath + amountProp.jsonPath,
//        unitAttribute: basePath + unitProp.jsonPath,

        initialValue: initial, basePath: paths.path,
      )
    ];
  }
}

enum DurationProperty { amount, unit }
