//ignore_file: unused_local_variable

import 'package:flutter/widgets.dart';
import 'package:sunny_dart/json.dart';
import 'package:sunny_dart/sunny_dart.dart';
import 'package:sunny_forms/form_fields/image_url_picker.dart';
import 'package:sunny_forms/meta/meta_property_handler.dart';
import 'package:sunny_forms/sunny_form_controller.dart';
import 'package:sunny_forms/sunny_form_field.dart';
import 'package:sunny_forms/sunny_hud.dart';
import 'package:sunny_sdk_core/sunny_sdk_core.dart';
import 'package:sunny_service_stubs/models.dart';
import 'package:sunny_service_stubs/options.dart';

class ImageUploadPropertyHandler extends MetaPropertyHandler<IImageContent>
    with LoggingMixin {
  ImageUploadPropertyHandler();

  /// Renders a widget (or null) for a provided [HandledPaths] (see [acceptProperties])
  Iterable<Widget> renderProperty(
      BuildContext context,
      IMetaFormContext metaForm,
      HandledPaths paths,
      SunnyFormFieldState<IImageContent> state,
      {Key? key}) {
    final fullPath = paths.fullPath<IImageContent>();
    final form = context.form();
    final initialValue = form.path(fullPath);
    final readyCheck = state.form?.requestReadyCheck(fullPath);
    return [
      ImageUrlControl(
        size: 50,
        initialValue: initialValue?.imageUrl?.toUri(),
        type: 'picture',
        readyCheck: readyCheck,
        allowThirdParty: true,
        onUploaded: (upload) {
          SunnyHud.info(context, "Uploaded Successfully", duration: 1500.ms);
          final imageContent = state.value ??
              IImageContent.of(
                dismissible: false,
                delivery: IContentDelivery.inline_,
                imageUrl: null,
              );
          imageContent.imageUrl = upload.toString();
          state.updateValue(imageContent, AttributeSource.control);
        },
        onFileChanged: (file) {
          // ignore: missing_required_param, unused_local_variable
          final imageContent = state.value ??
              IImageContent.of(
                dismissible: false,
                delivery: IContentDelivery.inline_,
                imageUrl: null,
              );
          imageContent.selectedFile = file;
        },
        onError: (response, stack) {
          SunnyHud.error(context, "Problem uploading", duration: twoSeconds);
          log.warning("Error uploading: $response", response, stack);
          state.error = ValidationError.ofString(
            fullPath,
            "Problem uploading",
            debugMessage: "$response",
          );
        },
      )
    ];
  }

  @override
  Iterable<HandledPaths> acceptProperties(
      JsonPath? basePath,
      IMSchemaDefinition definition,
      Map<JsonPath, IMSchemaProperty> available) {
    return available
        .filterEntries((k, v) => v.baseCode == ImageContentRef.baseCode)
        .mapEntries((k, v) => HandledPaths.ofProperty(v, basePath!, this));
  }
}
