import 'package:flutter/widgets.dart';
import 'package:meta_forms/meta_forms.dart';

import 'package:sunny_dart/json.dart';
import 'package:sunny_forms/form_fields/select_field.dart';
import 'package:sunny_forms/meta/meta_property_handler.dart';
import 'package:sunny_forms/options/options_service.dart';
import 'package:sunny_forms/options/options_view_service.dart';
import 'package:sunny_forms/sunny_form_field.dart';
import 'package:sunny_sdk_core/sunny_sdk_core.dart';
import 'package:sunny_service_stubs/options.dart';

class MverseLinkOptionHandler<T extends MModel>
    extends MetaPropertyHandler<MKey?> {
  MverseLinkOptionHandler();

  @override
  Iterable<Widget> renderProperty(
      BuildContext context,
      IMetaFormContext metaForm,
      HandledPaths paths,
      SunnyFormFieldState<MKey?> state,
      {Key? key}) {
    final prop = paths.property as IRefProperty;
    final theRef = prop.theRef;
    final _mfProp = metaForm.prop(paths.property!);
    KeyedOptionsHandler<MKey?, T?> handler =
        context.optionsService.getKeyedHandler(theRef, metaContext: _mfProp);

    final viewHandler =
        context.typeaheadService.getTypeaheadRenderer<MKey, T>(theRef);

    final fullPath = paths.fullPath<MKey>();

    return [
      OptionSelectorControl<MKey?, T?>(
        key: Key("field-$fullPath"),
        handler: handler,
        focusNode: state.focusNode,
        isRequired: prop.isRequired == true,
//      noOptionsLabel: widget.noOptionsLabel,
//        autofocus: widget.autofocus ?? false,
        onChange: (value, source) {
          state.updateValue(value?.key, source);
        },
        expected: (option) => option?.key,
        onError: (error) => state.error = error,
        initialKeyValue: state.value,
        viewHandler: viewHandler,
      )
    ];
  }

  @override
  Iterable<HandledPaths> acceptProperties(
      JsonPath? basePath,
      IMSchemaDefinition definition,
      Map<JsonPath, IMSchemaProperty> available) {
    return [
      for (final v in available.values)
        if (v is IRefProperty) HandledPaths.ofProperty(v, basePath!, this),
    ];
  }
}
