//ignore_file: unused_local_variable

import 'package:flutter/widgets.dart';
import 'package:sunny_dart/json.dart';
import 'package:sunny_dart/sunny_dart.dart';
import 'package:sunny_forms/form_fields/media_gallery.dart';
import 'package:sunny_forms/media/media_content_type.dart';
import 'package:sunny_forms/meta/meta_property_handler.dart';
import 'package:sunny_forms/sunny_form_controller.dart';
import 'package:sunny_forms/sunny_form_field.dart';
import 'package:sunny_forms/sunny_hud.dart';
import 'package:sunny_sdk_core/sunny_sdk_core.dart';
import 'package:sunny_service_stubs/models.dart';
import 'package:sunny_service_stubs/options.dart';

class VideoGalleryPropertyHandler
    extends MetaPropertyHandler<List<IVideoContent>> with LoggingMixin {
  VideoGalleryPropertyHandler();

  /// Renders a widget (or null) for a provided [HandledPaths] (see [acceptProperties])
  Iterable<Widget> renderProperty(
      BuildContext context,
      IMetaFormContext metaForm,
      HandledPaths paths,
      SunnyFormFieldState<List<IVideoContent?>> state,
      {Key? key}) {
    final fullPath = paths.fullPath<List<IVideoContent>>();
    final form = context.form();
    final initialValue = form.path(fullPath);
    final readyCheck = state.form?.requestReadyCheck(fullPath);
    return [
      MediaGalleryControl<IVideoContent?>(
        name: "$fullPath",
        size: 50,
        columns: 1,
        itemHeight: 200.0,
        initialValue: initialValue,
        type: MediaContentType.video,
        progress: readyCheck,
        // readyCheck: readyCheck,
        allowThirdParty: true,
        onChanged: (files) {
          final videoContent = files;
          state.updateValue(videoContent, AttributeSource.control);
        },
        onError: (response, stack) {
          SunnyHud.error(context, "Problem uploading", duration: twoSeconds);
          log.warning("Error uploading: $response", response, stack);
          state.error = ValidationError.ofString(
            fullPath,
            "Problem uploading",
            debugMessage: "$response",
          );
        },
      )
    ];
  }

  @override
  Iterable<HandledPaths> acceptProperties(
      JsonPath? basePath,
      IMSchemaDefinition definition,
      Map<JsonPath, IMSchemaProperty> available) {
    return available.filterEntries((k, v) {
      return v is IListProperty &&
          v.allItems?.baseCode == VideoContentRef.baseCode;
    }).mapEntries((k, v) => HandledPaths.ofProperty(v, basePath!, this));
  }
}
