// import 'dart:async';
//
// import 'package:flutter/widgets.dart';
// import 'package:sunny_dart/time.dart';
// import 'package:sunny_forms/model/smart_date_query.dart';
// import 'package:sunny_forms/shared/shared.dart';
// import 'package:timezone/timezone.dart';
//
// typedef QueryInterpolator = String Function(String input, String date);
//
// /// Used to make smartDate queries that are based on fact data.
// class FactSmartDateQuerySource implements SmartDateQuerySource {
//   /// The query that will be sent to the smartDate service.  May include interpolation to replace fact tokens with
//   /// actual date terms, eg.  a week before her anniversary -> a week before 2007-03-10
//
//   final FactSchema factSchema;
//   final FactMetaDate metaDate;
//   final Contact contact;
//   final Fact fact;
//
//   /// We found an actual date mapped to a search term, eg "birthday", "joined date", "wedding", "kid's birthday", etc
//   /// The final date may be a derivative of this date (before, after, etc).
//   final TZDateTime resolvedDate;
//
//   /// We may have found an actual flexiDate as well.
//   final FlexiDate resolvedFlexiDate;
//
//   /// The query we're sending to the smart date parser.  This will probably be doctored to interpolate the resolved
//   /// date, since the smart date parser on the server doesn't understand [Fact] semantics
//   final String smartDateQuery;
//
//   /// If this query is completed and selected, this is what should show as the logical description of the date
//   final String selection;
//
//   FactSmartDateQuerySource.ofSchemaQuery(FactDateSchemaQuery schemaQuery, this.contact,
//       {this.resolvedDate, this.resolvedFlexiDate})
//       : assert(schemaQuery.fact != null, "Should have a fact"),
//         factSchema = schemaQuery.factSchema,
//         metaDate = schemaQuery.metaDate,
//         selection = schemaQuery.buildSelectionString(resolvedDate),
//         fact = schemaQuery.fact,
//         smartDateQuery = schemaQuery.buildQuery(resolvedDate);
//
//   FactSmartDateQuerySource.ofMetaDate(
//     FactSchema factSchema,
//     FactMetaDate metaDate,
//     Fact fact,
//     Contact contact, {
//     TZDateTime resolvedDate,
//     FlexiDate resolvedFlexiDate,
//     String originalQuery,
//     String matchedTerm,
//   }) : this(factSchema, metaDate, fact, resolvedFlexiDate, resolvedDate, contact,
//             smartDateQuery: originalQuery?.replaceAll(matchedTerm, resolvedDate.resolvedDateString),
//             selection: originalQuery?.replaceAll(matchedTerm, metaDate.remindLabel));
//
//   FactSmartDateQuerySource(
//       this.factSchema, this.metaDate, this.fact, this.resolvedFlexiDate, this.resolvedDate, this.contact,
//       {this.smartDateQuery, this.selection})
//       : assert(metaDate != null),
//         assert(fact != null, "Should have a fact"),
//         assert(resolvedDate != null);
//
//   @override
//   List<SmartDateQueryResult> toQueryResult(SmartDate result) {
//     return [
//       ...result.dates.map((date) => FactSmartDateQueryResult.ofQuerySource(this, date, null, result)),
//     ];
//   }
//
//   /// Converts this query source immediately into a query result.  This is for the case that we're not sending the
//   /// date query to the nlp engine, but just want to use the resolved date completely.
//   SmartDateQueryResult toSingleQueryResult() {
//     return FactSmartDateQueryResult.ofQuerySource(this, resolvedDate, resolvedFlexiDate);
//   }
// }
//
// /// The result of a smartDate query that was based on a fact.
// class FactSmartDateQueryResult with DefaultQueryResultMixin, ListableMixin implements SmartDateQueryResult {
//   final FactSmartDateQuerySource source;
//   final DateTime resultDate;
//   final FlexiDate flexiDate;
//   final bool isFlexible;
//   final bool isTimeInferred;
//   final String selection;
//   final TimeSpan recurrence;
//   final bool isRecurring;
//
//   String overrideTitle;
//
//   FactSmartDateQueryResult.ofMetaData(
//     FactSchema schema,
//     FactMetaDate metaDate,
//     Fact fact,
//     Contact contact, {
//     TZDateTime resultDate,
//     Location location,
//   }) : this._(
//           schema,
//           metaDate,
//           fact,
//           contact,
//           resultDate: resultDate ?? fact.getDate(metaDate, location ?? sunnyLocalization.userLocation),
//           selection: smartDateFormat(resultDate ?? fact.getDate(metaDate, location ?? sunnyLocalization.userLocation),
//               formatTime: metaDate.hasTime),
//           isTimeInferred: !metaDate.hasTime,
//         );
//
//   FactSmartDateQueryResult._(
//     FactSchema schema,
//     FactMetaDate metaDate,
//     Fact fact,
//     Contact contact, {
//     this.isRecurring,
//     DateTime resultDate,
//     FlexiDate flexiDate,
//     this.selection,
//     this.recurrence,
//     this.isTimeInferred = true,
//   })  : assert(fact != null),
//         assert(metaDate != null),
//         assert(schema != null),
//         assert(resultDate != null || flexiDate != null),
//         isFlexible = flexiDate != null,
//         flexiDate = flexiDate ?? FlexiDate.fromDateTime(resultDate),
//         resultDate = resultDate ?? flexiDate.toDateTime(),
//         source = FactSmartDateQuerySource.ofMetaDate(
//           schema,
//           metaDate,
//           fact,
//           contact,
//           resolvedFlexiDate: flexiDate ?? FlexiDate.fromDateTime(resultDate),
//           resolvedDate: resultDate.withTimeZone(),
//         );
//
//   FactSmartDateQueryResult.ofQuerySource(this.source, this.resultDate, FlexiDate flexiDate, [SmartDate smartDate])
//       : isTimeInferred = (smartDate?.isTimeInferred ?? resultDate?.hasTime?.negate) == true,
//         recurrence = smartDate?.toTimeSpan(),
//         isFlexible = flexiDate != null,
//         flexiDate = flexiDate ?? FlexiDate.fromDateTime(resultDate),
//         isRecurring = smartDate?.isRecurring ?? false,
//         selection = source.selection;
//
//   @override
//   String get title => overrideTitle ?? source.fact.title(source.contact);
//
//   @override
//   List get subtitle => [
//         if (source.fact is RemindableFact) (source.fact as RemindableFact).remindLabel(source.metaDate),
//         if (source.fact is! RemindableFact) source.fact.subtitle(source.contact),
//         "Due date: $selection",
//       ];
//
//   String get remindLabel {
//     if (source.fact is RemindableFact) {
//       return (source.fact as RemindableFact).remindLabel(source.metaDate);
//     } else {
//       return source.fact.subtitle(source.contact);
//     }
//   }
//
//   @override
//   String get icon => source.factSchema.icon;
//
//   @override
//   bool get isResolved => true;
//
//   @override
//   String get smartDateQuery => source.smartDateQuery;
//
//   @override
//   TaskDueConfig toDueConfig() {
//     return MetaDateDueConfig.of(schemaRef: source.factSchema.ref, path: source.metaDate.path, factId: source.fact?.id);
//   }
// }
//
// /// This query wasn't sent to the server, but still represents something we want to show the user.  This means
// /// that we found a fact name, but the focused contact (or other scope) doesn't contain an actual date.
// ///
// /// The user will likely be prompted to add the appropriate date
// class UnresolvedFactDateQueryResult with ListableMixin, DefaultQueryResultMixin implements SmartDateQueryResult {
//   final FactSchema factSchema;
//   final FactMetaDate metaDate;
//   final String smartDateQuery;
//   final Contact contact;
//   final String originalQuery;
//   final String matchedTerm;
//
//   UnresolvedFactDateQueryResult.ofSchemaQuery(FactDateSchemaQuery source, Contact contact)
//       : this(source.factSchema, source.metaDate, contact,
//             originalQuery: source.originalQuery,
//             matchedTerm: source.matchedTerm,
//             smartDateQuery: source.originalQuery.replaceAll(source.matchedTerm, source.metaDate.remindLabel));
//
//   UnresolvedFactDateQueryResult(
//     this.factSchema,
//     this.metaDate,
//     this.contact, {
//     this.originalQuery,
//     this.matchedTerm,
//     this.smartDateQuery,
//   })  : assert(contact != null),
//         assert(metaDate != null),
//         assert(factSchema != null);
//
//   @override
//   String get icon => factSchema.icon;
//
//   @override
//   bool get isResolved => false;
//
//   @override
//   FlexiDate get flexiDate => null;
//
//   @override
//   bool get isFlexible => false;
//
//   @override
//   String get selection => metaDate.remindableInfo.remindMeLabel;
//
//   @override
//   TimeSpan get recurrence {
//     /// Returns the best recurring pattern for this
//     if (metaDate.isRecurring) {
//       return TimeSpan(
//           years: metaDate.recurs == FactRecurrence.yearly_ ? 1 : 0,
//           months: metaDate.recurs == FactRecurrence.monthly_ ? 1 : 0);
//     } else if (metaDate.isHistorical) {
//       return TimeSpan(years: 1);
//     } else {
//       return null;
//     }
//   }
//
//   @override
//   bool get isRecurring => null;
//
//   @override
//   String get title => "${metaDate.remindLabel}";
//
//   @override
//   List get subtitle => null;
//
// //  String get subtitle => "${contact.firstName} does not have this information yet";
//
//   @override
//   DateTime get resultDate => null;
//
//   @override
//   bool get isTimeInferred => false;
//
//   @override
//   Widget buildListTile(BuildContext context, {bool isCard, VoidCallback onTap}) =>
//       toListTile(context, isCard: isCard, onTap: onTap);
//
//   @override
//   FutureOr<SmartDateQueryResult> onSelected(BuildContext context) async {
//     return this;
//   }
//
// //  @override
// //  FutureOr<SmartDateQueryResult> onTap(BuildContext context) async {
// //    final emptyFact = mmodelRegistry.instantiate(json: <String, dynamic>{}, type: factSchema.ref);
// //    final savedFact = await showFactCreatePage<Fact>(context, contact: contact, schema: factSchema, fact: emptyFact);
// //    final location = await SunnyLocalization.userLocationAsync;
// //    if (savedFact != null) {
// //      final savedDate = savedFact.getDate(metaDate, location);
// //      final querySource = FactSmartDateQuerySource.ofMetaDate(factSchema, metaDate, savedFact, savedDate,
// //          originalQuery: originalQuery, matchedTerm: matchedTerm);
// //      assert(querySource != null);
// //      final result = await smartDateService.executeQueries([querySource], location: location);
// //
// //      /// I guess we just pick the first one?  Not likely that multiple results will be helpful here...
// //      return result.firstOrNull();
// //    } else {
// //      return null;
// //    }
// //  }
//
//   @override
//   TaskDueConfig toDueConfig() {
//     return MetaDateDueConfig.of(schemaRef: factSchema.ref, path: metaDate.path);
//   }
// }
//
// extension ResolvedFactDateExtensions on FactDateSchemaQuery {
//   FactSmartDateQuerySource toSmartDateQuery(Contact contact, Location location) {
//     assert(fact != null);
//     assert(metaDate != null);
//
//     final resolvedDate = fact.getDate(metaDate, location);
//     final resolvedFlexiDate = fact.getFlexiDate(metaDate);
//     if (resolvedDate != null || resolvedFlexiDate != null) {
//       return FactSmartDateQuerySource.ofSchemaQuery(
//         this,
//         contact,
//         resolvedDate: resolvedDate,
//         resolvedFlexiDate: resolvedFlexiDate,
//       );
//     } else {
//       return null;
//     }
//   }
// }
//
// extension _DateTimeFactExtension on DateTime {
//   String get resolvedDateString => "$year-$month-$day";
// }
//
// extension FactQueryExtension on FactDateSchemaQuery {
//   /// Creates the readable selection string
//   String buildSelectionString(TZDateTime resolvedDate) {
//     // Replace matched tokens with the name of the actual date field
//     return originalQuery.replaceAll(matchedTerm, metaDate.remindLabel);
//   }
//
//   /// Creates the query to be sent to SmartDate (natty)
//   String buildQuery(TZDateTime resolvedDate) => originalQuery.replaceAll(matchedTerm, resolvedDate.resolvedDateString);
// }
