// import 'dart:async';
//
// import 'package:flutter/widgets.dart';
// import 'package:sunny_dart/sunny_dart.dart';
// import 'package:sunny_dart/time.dart';
// import 'package:sunny_forms/icon/named_icon.dart';
// import 'package:sunny_forms/model/smart_date_due_config.dart';
// import 'package:sunny_forms/model/smart_date_fact_query.dart';
// import 'package:sunny_forms/model/task_due_config.dart';
// import 'package:sunny_forms/shared/flexi_date.dart';
// import 'package:sunny_forms/shared/listable.dart';
// import 'package:sunny_forms/shared/select_option.dart';
// import 'package:sunny_forms/smart_dates/smart_date.dart';
// import 'package:sunny_forms/smart_dates/smart_date_format.dart';
//
// /// Represents a query to be sent to the [SmartDateService] - because of [Fact] dates, we may do some pre-processing
// /// to the query, which is why we subclass.
// abstract class SmartDateQuerySource {
//   String get smartDateQuery;
//
//   List<SmartDateQueryResult> toQueryResult(SmartDate result);
// }
//
// /// Result of a date query - might actually be an unresolved date, like "This contact doesn't have an anniversary"
// abstract class SmartDateQueryResult {
//   /// The query that was sent to the [SmartDateService]
//   String get smartDateQuery;
//
//   /// Whether the query resolved to an actual date
//   bool get isResolved;
//
//   /// The resolved date
//   DateTime get resultDate;
//
//   /// The resolved date, as a [FlexiDate]
//   FlexiDate get flexiDate;
//
//   /// Whether the result is a [FlexiDate].  If true, then [flexiDate] should not be null.  If false, then [resultDate]
//   /// should not be null
//   bool get isFlexible;
//
//   /// Whether the result date has an inferred time.
//   bool get isTimeInferred;
//
//   /// What gets put back into the search box when this item is selected
//   String get selection;
//
//   /// Returns the suggested recurrence for this result.  This doesn't necessarily
//   /// mean that the result should be interpreted as recurring.  See [isRecurring]
//   TimeSpan get recurrence;
//
//   /// Whether or not we should automatically set this to recurring
//   bool get isRecurring;
//
//   factory SmartDateQueryResult.ofDateTime(
//     DateTime dateTime, {
//     String icon,
//     String overrideLabel,
//   }) {
//     return TermResult.ofDateTime(dateTime,
//         icon: icon, overrideTitle: overrideLabel);
//   }
//
//   factory SmartDateQueryResult.ofFlexiDate(
//     FlexiDate flexiDate, {
//     String icon,
//     String overrideLabel,
//   }) {
//     assert(flexiDate != null);
//     return TermResult.ofFlexiDate(flexiDate,
//         icon: icon, overrideTitle: overrideLabel);
//   }
//
//   FutureOr<SmartDateQueryResult> onSelected(BuildContext context);
//
//   Widget buildListTile(BuildContext context, {bool isCard, VoidCallback onTap});
//
//   TaskDueConfig toDueConfig();
//
//   Option<SmartDateQueryResult> toOption();
// }
//
// /// Default impl for [SmartDateQuerySource]
// class TermSource implements SmartDateQuerySource {
//   @override
//   final String smartDateQuery;
//
//   final bool isFlexiDateQuery;
//
//   TermSource(this.smartDateQuery, {this.isFlexiDateQuery = false})
//       : assert(isFlexiDateQuery != null);
//
//   @override
//   List<SmartDateQueryResult> toQueryResult(SmartDate result) {
//     return [
//       ...result.dates.map((date) {
//         return TermResult.ofSmartDate(this, result, date);
//       }),
//     ];
//   }
// }
//
// /// Default impl of [SmartDateQueryResult]
// class TermResult
//     with DefaultQueryResultMixin, ListableMixin
//     implements SmartDateQueryResult {
//   final bool isTimeInferred;
//   final DateTime resultDate;
//   final FlexiDate flexiDate;
//   String icon;
//   final bool isFlexible;
//   final SmartDateQuerySource source;
//   final bool isRecurring;
//   final TimeSpan recurrence;
//
//   String overrideTitle;
//
//   TermResult.ofSmartDate(
//     this.source,
//     SmartDate smartDate,
//     this.resultDate, {
//     this.icon,
//     this.overrideTitle,
//   })  : flexiDate = FlexiDate.fromDateTime(resultDate),
//         isFlexible = false,
//         isTimeInferred = smartDate.isTimeInferred,
//         isRecurring = smartDate.isRecurring ||
//             source.smartDateQuery?.toLowerCase()?.startsWith("every") == true,
//         recurrence = smartDate.toTimeSpan(),
//         smartDateQuery = source.smartDateQuery;
//
//   TermResult.ofDateTime(
//     this.resultDate, {
//     this.icon,
//     this.isRecurring,
//     this.recurrence,
//     this.overrideTitle,
//   })  : assert(resultDate != null),
//         flexiDate = FlexiDate.fromDateTime(resultDate),
//         isFlexible = false,
//         smartDateQuery = null,
//         source = null,
//         isTimeInferred = resultDate.hasTime;
//
//   TermResult.ofFlexiDate(
//     this.flexiDate, {
//     this.icon,
//     this.isRecurring,
//     this.recurrence,
//     this.overrideTitle,
//   })  : assert(flexiDate != null),
//         smartDateQuery = null,
//         resultDate = flexiDate.toDateTime(),
//         source = null,
//         isFlexible = true,
//         isTimeInferred = true;
//
//   @override
//   String get selection => isFlexible
//       ? flexiDate.formatted()
//       : smartDateFormat(resultDate, formatTime: this.hasTime);
//
//   @override
//   bool get isResolved => true;
//
//   @override
//   String get title => overrideTitle;
//
//   @override
//   final String smartDateQuery;
//
//   @override
//   TaskDueConfig toDueConfig() {
//     return SmartDateDueConfig.of(
//         hasTime: this.hasTime,
//         originalQuery: smartDateQuery,
//         interpolatedQuery: selection,
//         matchedTerm: null);
//   }
// }
//
// extension SmartDateQueryResultExtension on SmartDateQueryResult {
//   bool get shouldDisplayTime =>
//       resultDate?.hasTime == true &&
//       isTimeInferred != true &&
//       resultDate.sinceNow().inSeconds.abs() > 10;
//
//   String get formattedDate {
//     if (isFlexible) return flexiDate.formatted();
//     final localDate = resultDate.toLocal();
//     var showTime = shouldDisplayTime;
//     return smartDateFormat(localDate, formatTime: showTime);
//   }
// }
//
// mixin DefaultQueryResultMixin implements SmartDateQueryResult, Listable {
//   FutureOr<SmartDateQueryResult> onSelected(BuildContext context) => this;
//
//   Option<SmartDateQueryResult> toOption() {
//     final self = this;
//     if (self is FactSmartDateQueryResult) {
//       return Option(
//         this,
//         label: self.remindLabel,
//         icon: SunnyIcons.getIconOrNull(this.icon)?.toPrefixIcon(),
//         subtitle: formattedDate.toStringList(),
//         selection: self.remindLabel,
//       );
//     } else if (self is UnresolvedFactDateQueryResult) {
//       return Option(
//         this,
//         label: self.metaDate.remindLabel.capitalize(),
//         icon: NamedIcons[this.icon]?.toPrefixIcon(),
//         subtitle: const ["Hasn't been set"],
//         selection: self.metaDate.remindLabel,
//       );
//     } else if (self is TermResult) {
//       if (self.smartDateQuery.isNotNullOrBlank) {
//         return Option(
//           this,
//           label: self.smartDateQuery,
//           icon: SunnyIcons.getIconOrNull(this.icon)?.toPrefixIcon(),
//           subtitle: [self.formattedDate],
//           selection: self.formattedDate,
//         );
//       } else {
//         return Option(
//           this,
//           label: self.formattedDate,
//           icon: SunnyIcons.getIconOrNull(this.icon)?.toPrefixIcon(),
//           selection: self.formattedDate,
//         );
//       }
//     } else {
//       final title = this.title ?? formattedDate;
//       return Option(
//         this,
//         label: this.title ?? smartDateQuery ?? "",
//         icon: SunnyIcons.getIconOrNull(this.icon)?.toPrefixIcon(),
//         subtitle: this.formattedDate.toStringList(),
//         selection: title,
//       );
//     }
//   }
//
//   Widget buildListTile(BuildContext context,
//       {bool isCard, VoidCallback onTap}) {
//     return PlatformListTile(
//         margin: isCard != true ? EdgeInsets.zero : null,
//         dense: false,
//         leading: SunnyIcons.getIconOrNull(this.icon)?.toListSelectOption(),
//         card: isCard ?? this.isCard,
//         onTap: onTap,
//         title: TileTitle(this.title ?? formattedDate),
//         subtitle: subtitle.toColumn(),
//         trailing: textOrNull(trailing));
//   }
//
//   String get selection => isFlexible ? flexiDate.formatted() : formattedDate;
//
//   bool get hasTime {
//     if (isTimeInferred == true) return false;
//     return resultDate?.hasTime;
//   }
// }
//
// /// Used when you need to wrap a result, but delegate changes
// class DelegatingSmartDateQueryResult implements SmartDateQueryResult {
//   final SmartDateQueryResult delegate;
//   final String _title;
//   final String _selection;
//   final List _subtitle;
//   final String _icon;
//
//   @override
//   Widget buildListTile(BuildContext context,
//       {bool isCard, VoidCallback onTap}) {
//     return delegate.buildListTile(context, isCard: isCard, onTap: onTap);
//   }
//
//   @override
//   FlexiDate get flexiDate => delegate.flexiDate;
//
//   @override
//   bool get isFlexible => delegate.isFlexible;
//
//   @override
//   bool get isRecurring => delegate.isRecurring;
//
//   @override
//   bool get isResolved => delegate.isResolved;
//
//   @override
//   bool get isTimeInferred => delegate.isTimeInferred;
//
//   @override
//   FutureOr<SmartDateQueryResult> onSelected(BuildContext context) {
//     return delegate.onSelected(context);
//   }
//
//   @override
//   TimeSpan get recurrence => delegate.recurrence;
//
//   @override
//   DateTime get resultDate => delegate.resultDate;
//
//   @override
//   String get selection => delegate.selection;
//
//   @override
//   String get smartDateQuery => delegate.smartDateQuery;
//
//   @override
//   TaskDueConfig toDueConfig() {
//     return delegate.toDueConfig();
//   }
//
//   @override
//   Option<SmartDateQueryResult> toOption() {
//     final option = delegate.toOption();
//     return Option<SmartDateQueryResult>(delegate,
//         label: _title ?? option.label ?? _title,
//         selection: _selection ?? option.selection,
//         icon: SunnyIcons.getIconOrNull(_icon)?.toPrefixIcon() ?? option.icon,
//         subtitle: _subtitle.ifEmpty(() => option.subtitle)?.toList(),
//         extraTokens: option.extraTokens);
//   }
//
//   SmartDateQueryResult unbox([List<SmartDateQueryResult> processed]) {
//     processed ??= [];
//     final _ = delegate;
//     if (_ is DelegatingSmartDateQueryResult) {
//       if (processed.contains(_)) {
//         throw "Recursion error";
//       }
//       processed.add(this);
//       return _.unbox(processed);
//     } else {
//       return _;
//     }
//   }
//
//   DelegatingSmartDateQueryResult(
//     final SmartDateQueryResult delegate, {
//     String title,
//     List subtitle,
//     String icon,
//     String selection,
//   })  : delegate = (delegate is DelegatingSmartDateQueryResult
//             ? delegate.unbox()
//             : delegate),
//         _title = title,
//         _subtitle = subtitle,
//         _selection = selection,
//         _icon = icon;
// }
