import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:sunny_dart/json.dart';

class Notifier {
  final List<VoidCallback> _listeners = [];

  addListener(VoidCallback listener) {
    if (!_listeners.contains(listener)) {
      _listeners.add(listener);
    }
  }

  removeListener(VoidCallback listener) {
    _listeners.remove(listener);
  }

  notify() {
    _listeners.forEach((fn) => fn());
  }
}

class VNotifier<T> {
  List<ValueAccepter<T>> _listeners = [];

  addListener(ValueAccepter<T> listener) {
    if (!_listeners.contains(listener)) {
      _listeners.add(listener);
    }
  }

  removeListener(ValueAccepter<T> listener) {
    _listeners.remove(listener);
  }

  dispose() {
    _listeners.clear();
  }

  notify(T value) {
    _listeners.forEach((fn) => fn(value));
  }

  call(T value) {
    notify(value);
  }
}

typedef ValueAccepter<T> = void Function(T input);
typedef AttributeAccepter<T> = void Function(JsonPath path, T input);
typedef FieldConverter<T> = T Function(String? input);
typedef FieldTransformer<F, T> = T Function(F from);

//ValueNotifier<T> transform<F, T>(ValueNotifier<F> notifier, ValueTransformer<T, F> transformer) {
//  ValueNotifier<T> to = ValueNotifier<T>(transformer(notifier.value));
//
//  notifier.addListener(() {
//    to.value = transformer(notifier.value);
//  });
//  return to;
//}
