/*
 * flutter_platform_widgets
 * Copyright (c) 2018 Lance Johnstone. All rights reserved.
 * See LICENSE for distribution and usage details.
 */

import 'package:flutter/cupertino.dart' show CupertinoActionSheetAction;
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart'
    show
        // ignore: deprecated_member_use
        FlatButton,
        Brightness,
        ButtonTextTheme,
        MaterialTapTargetSize;
import 'package:flutter/widgets.dart';
import 'package:flutter_platform_widgets/flutter_platform_widgets.dart';

abstract class _BaseData {
  _BaseData({this.child, this.onPressed});

  final Widget? child;
  final VoidCallback? onPressed;
}

class MaterialBottomSheetActionData extends _BaseData {
  MaterialBottomSheetActionData({
    Widget? child,
    VoidCallback? onPressed,
    this.widgetKey,
    this.color,
    this.colorBrightness,
    this.disabledColor,
    this.disabledTextColor,
    this.highlightColor,
    this.onHighlightChanged,
    this.padding,
    this.shape,
    this.splashColor,
    this.textColor,
    this.textTheme,
    this.clipBehavior,
    this.materialTapTargetSize,
  }) : super(child: child, onPressed: onPressed);

  final Key? widgetKey;
  final Color? color;
  final Brightness? colorBrightness;
  final Color? disabledColor;
  final Color? disabledTextColor;
  final Color? highlightColor;
  final ValueChanged<bool>? onHighlightChanged;
  final EdgeInsetsGeometry? padding;
  final ShapeBorder? shape;
  final Color? splashColor;
  final Color? textColor;
  final ButtonTextTheme? textTheme;
  final Clip? clipBehavior;
  final MaterialTapTargetSize? materialTapTargetSize;
}

class CupertinoBottomSheetActionData extends _BaseData {
  CupertinoBottomSheetActionData({
    Widget? child,
    VoidCallback? onPressed,
    this.isDefaultAction,
    this.isDestructiveAction,
  }) : super(child: child, onPressed: onPressed);

  final bool? isDefaultAction;
  final bool? isDestructiveAction;
}

class PlatformBottomSheetAction
    extends PlatformWidgetBase<CupertinoActionSheetAction, Widget> {
  final Key? widgetKey;
  final Widget child;
  final VoidCallback onPressed;
  final bool? isDefaultAction;

  final PlatformBuilder<MaterialBottomSheetActionData>? android;
  final PlatformBuilder<CupertinoBottomSheetActionData>? ios;

  PlatformBottomSheetAction({
    Key? key,
    this.widgetKey,
    this.isDefaultAction,
    required this.child,
    required this.onPressed,
    this.android,
    this.ios,
  }) : super(key: key);

  @override
  Widget createMaterialWidget(BuildContext context) {
    MaterialBottomSheetActionData? data;
    if (android != null) {
      data = android!(context, platform(context));
    }

    return Padding(
      padding: const EdgeInsets.all(8.0),
      // ignore: deprecated_member_use
      child: FlatButton(
        key: data?.widgetKey ?? widgetKey,
        child: data?.child ?? child,
        onPressed: data?.onPressed ?? onPressed,
        color: data?.color,
        colorBrightness: data?.colorBrightness ?? Brightness.light,
        disabledColor: data?.disabledColor,
        disabledTextColor: data?.disabledTextColor,
        highlightColor: data?.highlightColor,
        onHighlightChanged: data?.onHighlightChanged,
        padding: data?.padding,
        shape: data?.shape,
        splashColor: data?.splashColor,
        textColor: data?.textColor,
        textTheme: data?.textTheme,
        clipBehavior: data?.clipBehavior ?? Clip.none,
        materialTapTargetSize: data?.materialTapTargetSize,
      ),
    );
  }

  @override
  CupertinoActionSheetAction createCupertinoWidget(BuildContext context) {
    CupertinoBottomSheetActionData? data;
    if (ios != null) {
      data = ios!(context, platform(context));
    }

    return CupertinoActionSheetAction(
      child: data?.child ?? child,
      isDefaultAction: this.isDefaultAction ?? data?.isDefaultAction ?? false,
      isDestructiveAction: data?.isDestructiveAction ?? false,
      onPressed: data?.onPressed ?? onPressed,
    );
  }
}
