import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter_platform_widgets/flutter_platform_widgets.dart';

class PlatformFlatButton extends PlatformWidgetBase {
  final Key? widgetKey;

  final VoidCallback? onPressed;
  final Widget? child;
  final Color? color;
  final EdgeInsetsGeometry? padding;
  final Color? disabledColor;
  final Color? disabledTextColor;

  final PlatformBuilder<MaterialRaisedButtonData>? android;
  final PlatformBuilder<CupertinoButtonData>? ios;

  PlatformFlatButton(
      {Key? key,
      this.widgetKey,
      this.child,
      this.onPressed,
      this.color,
      this.disabledColor,
      this.disabledTextColor,
      this.padding,
      this.android,
      this.ios})
      : super(key: key);

  @override
  // ignore: deprecated_member_use
  FlatButton createMaterialWidget(BuildContext context) {
    MaterialRaisedButtonData? data;
    if (android != null) {
      data = android!(context, platform(context));
    }

    // ignore: deprecated_member_use
    return FlatButton(
      key: data?.widgetKey ?? widgetKey,
      child: data?.child ?? child!,
      onPressed: data?.onPressed ?? onPressed,
      color: data?.color ?? color,
      colorBrightness: data?.colorBrightness,
      disabledColor: data?.disabledColor ?? disabledColor,
      disabledTextColor: data?.disabledTextColor ?? disabledTextColor,
      highlightColor: data?.highlightColor,
      onHighlightChanged: data?.onHighlightChanged,
      padding: data?.padding ?? padding,
      shape: data?.shape,
      splashColor: data?.splashColor,
      textColor: data?.textColor,
      textTheme: data?.textTheme,
      clipBehavior: data?.clipBehavior ?? Clip.none,
      materialTapTargetSize: data?.materialTapTargetSize,
    );
  }

  @override
  CupertinoButton createCupertinoWidget(BuildContext context) {
    CupertinoButtonData? data;
    if (ios != null) {
      data = ios!(context, platform(context));
    }

    return CupertinoButton(
      //key: widgetKey,
      child: data?.child ?? child!,
      onPressed: data?.onPressed ?? onPressed,
      borderRadius:
          data?.borderRadius ?? const BorderRadius.all(Radius.circular(8.0)),
      color: data?.color ?? color,
      minSize: data?.minSize ?? 44.0,
      padding: data?.padding ?? padding,
      pressedOpacity: data?.pressedOpacity ?? 0.1,
      disabledColor: data?.disabledColor ??
          disabledColor ??
          CupertinoColors.quaternarySystemFill,
    );
  }
}
