import 'package:flutter/material.dart';
import 'package:flutter/painting.dart';
import 'package:flutter/widgets.dart';
import 'package:sunny_dart/info_x.dart';
import 'package:sunny_forms/dividing_line.dart';
import 'package:sunny_forms/sunny_colors.dart';

class SunnyListCell extends StatelessWidget {
  final EdgeInsets padding;
  final EdgeInsets margin;
  final Color color;
  final BorderRadius borderRadius;
  final bool useShadow;
  final double? height;
  final double? width;
  final Widget? child;
  final double? minHeight;
  final double? minWidth;
  final VoidCallback? onTap;
  final VoidCallback? onLongPress;
  final List<BoxShadow>? shadow;

  const SunnyListCell({
    Key? key,
    this.padding = SunnyMargins.listItemPadding,
    this.margin = SunnyMargins.listItemMargin,
    this.borderRadius = SunnyMargins.listItemRadius,
    this.color = Colors.white,
    this.height,
    this.width,
    this.child,
    this.useShadow = true,
    this.minWidth,
    this.minHeight,
    this.onTap,
    this.onLongPress,
    this.shadow,
  }) : super(key: key);

  const SunnyListCell.unpadded({
    Key? key,
    this.margin = SunnyMargins.listItemMargin,
    this.borderRadius = SunnyMargins.listItemRadius,
    this.color = Colors.white,
    this.height,
    this.width,
    this.child,
    this.useShadow = true,
    this.minWidth,
    this.minHeight,
    this.onTap,
    this.onLongPress,
    this.shadow,
  })  : padding = SunnyMargins.zero,
        super(key: key);

  @override
  Widget build(BuildContext context) {
    final shadows = shadow ??
        sunny.get<CupertinoVisualStyle>(context: context).cardShadow ??
        SunnyShadows.subtle;
    final widget = infoX.isIOS
        ? ConstrainedBox(
            constraints: BoxConstraints(
                minHeight: minHeight ?? 60,
                maxHeight: height ?? double.infinity,
                maxWidth: width ?? double.infinity,
                minWidth: minWidth ?? 60),
            child: Container(
              margin: margin,
              padding: padding,
              alignment: AlignmentDirectional.centerStart,
              decoration: BoxDecoration(
                  color: color,
                  borderRadius: borderRadius,
                  boxShadow: useShadow ? shadows : null),
              child: child,
            ),
          )
        : Card(
            margin: margin, child: Container(padding: padding, child: child));

    if (onTap != null || onLongPress != null) {
      return GestureDetector(
          onTap: onTap, onLongPress: onLongPress, child: widget);
    } else {
      return widget;
    }
  }
}
