import 'package:flutter/material.dart';
import 'package:overlay_support/overlay_support.dart';
import 'package:sunny_core_widgets/sunny_core_widgets.dart';
import 'package:sunny_dart/sunny_dart.dart';

typedef Widget OverlaySupportBuilder(
    BuildContext context, Getter<OverlaySupportEntry?> entry);
typedef void OverlaySupportCallback(
    BuildContext context, OverlaySupportEntry? entry);

var i = 0;

class SunnyNotificationDismissLink extends StatelessWidget {
  final Getter<OverlaySupportEntry>? entry;

  const SunnyNotificationDismissLink({Key? key, this.entry}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Tappable(
      child: Text("Dismiss", style: TextStyle(fontSize: 14)),
      onTap: (_) {
        entry!().dismiss();
      },
    );
  }
}

OverlaySupportEntry showNotification<T>(
    {String? title,
    Widget? titleWidget,
    Widget? body,
    bool isSwipeDismissable = true,
    Duration? duration,
    bool? showDismissLink,
    Widget? leading,
    OverlaySupportBuilder? trailing,
    OverlaySupportCallback? onTap}) {
  OverlaySupportEntry? entry;
  entry = showOverlayNotification(
    (context) {
      final overlay = context.themes.platformOverlayTheme;

      final child = Material(
        elevation: overlay.elevation,
        child: ListTileTheme(
          style: ListTileStyle.list,
          textColor: overlay.foregroundColor,
          iconColor: overlay.foregroundColor,
          dense: false,
          child: Container(
              color: overlay.backgroundColor,
              child: SafeArea(
                bottom: false,
                child: ListTile(
                    title: titleWidget ?? Text(title ?? "Notification"),
                    onTap: () {
                      onTap?.call(context, entry);
                    },
                    contentPadding: overlay.padding,
                    subtitle: body,
                    leading: leading,
                    trailing: trailing?.call(context, () => entry) ??
                        (showDismissLink == true
                            ? SunnyNotificationDismissLink(entry: () => entry!)
                            : null)),
              )),
        ),
      );
      return isSwipeDismissable
          ? Dismissible(
              direction: DismissDirection.up,
              onDismissed: (_) {
                entry?.dismiss(animate: false);
              },
              child: child,
              key: Key("overlay-${i++}"),
            )
          : child;
    },
    duration: duration ?? 0.seconds,
  );
  return entry;
}
