import 'package:flutter/widgets.dart';
import 'package:sunny_essentials/state/state_observer.dart';

abstract class SunnyRoutingWidget<R> implements Widget {
  void popRoute(BuildContext context, [R? result]);
}

abstract class SunnyStatelessWidget<R> extends StatelessWidget
    implements SunnyRoutingWidget<R> {
  void popRoute(BuildContext context, [R? result]) {
    Navigator.pop(context, result);
  }

  const SunnyStatelessWidget({Key? key}) : super(key: key);
}

mixin SunnyStatelessRoutingMixin<R> implements SunnyRoutingWidget<R> {
  void popRoute(BuildContext context, [R? result]) {
    Navigator.pop(context, result);
  }
}

/// Marker class that indicates what type of data is expected from a route pop
abstract class SunnyStatefulWidget<R> extends StatefulWidget
    with SunnyStatelessRoutingMixin<R> {
  @mustCallSuper
  const SunnyStatefulWidget({Key? key}) : super(key: key);
}

abstract class SunnyWidgetState<R, W extends SunnyStatefulWidget<R>>
    extends BaseState<W> {
  void popRoute([R? result, BuildContext? context]) {
    Navigator.pop(context ?? this.context, result);
  }
}
