import 'dart:ui';

import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter/widgets.dart';
import 'package:sunny_dart/sunny_dart.dart';

class SunnyColors {
  SunnyColors._();

  /* updated color specs as of 5/4/2019 */
  static const Color primaryAction = Color(0xffff5745);
  static const Color secondaryAction = Color(0xff00CECB);
  static const Color primaryText = Color(0xFF112141);
  static const Color subTitleText = Color(0xFF9B9BAC);
  static const Color indicatorText = Color(0xFF6D6D7A);
  static const Color disabledText = Color(0xFFD2D2DA);
  static const Color lightBorder = Color(0xFFEEEEEE);
  static const Color disabledNavIcon = Color(0xFFD2D2DA);
  static const Color background = Color(0xFFF9F9F9);
  static const Color darkerBackground = Color(0xFF9B9BAC);
  static const Color iconCheck = Color(0xFF00CECB);
  static const Color successGreen = Colors.green;
  static const Color defaultIcon = Color(0xFF1776D1);

  /* end color specs */

  //Lists
  static const Color swipePeek = Color(0xffff5745);
  static const Color swipeWarn = Colors.orange;
  static const Color favorite = Colors.yellow;
  static const Color swipeBlue = iconCheck;
  static const Color swipeGreen = Colors.green;

  // Borders and Shading
  static const Color bundleBorder = Color(0xffE5E5E5);
  static const Color bundleHeaderBackground = Color(0xffFF6E5E);

//  static const Color navBarBackground = Color(0xddffffff);
  static const Color navBarBackground = Color(0xddffffff);
  static const Color solidIcon = Color(0xff89889B);
  static const Color orangeShadow = Color(0xffB23C30);
  static const Color navBarInactive = Color(0xffC8C7D2);

  static const Color pageBackground = background;

  static const Color shadow = Color(0x11000000);
  static const Color darkerShadow = Color(0x33000000);

  static const Color actionCellBackground = disabledText;
  static const Color inputBackground = Color.fromRGBO(228, 225, 225, 1.0);

  static const Color inputControls = Color(0xffA8A6A6);
  static const Color inputBorder = Color(0x55A8A6A6);
  static const Color inputControlsFocus = Color(0xff6E6E6F);

  static const Color secondaryText = Color(0xff696875);
  static const Color tertiaryText = subTitleText;
  static const Color pageIndicatorInactive = Color.fromRGBO(0, 0, 0, 0.06);
  static const Color pageIndicator = Color(0xFF9B9BAC);
  static const Color white = Color(0xFFFFFFFF);
}

class SunnyShadows {
  SunnyShadows._();

  static const List<BoxShadow> subtle = [
    BoxShadow(
      color: SunnyColors.shadow,
      offset: Offset(3, 3),
      blurRadius: 10, // has the effect of softening the shadow
    )
  ];

  static const List<BoxShadow> defined = [
    BoxShadow(
      color: SunnyColors.shadow,
      offset: Offset(3, 3),

      blurRadius: 5, // has the effect of softening the shadow
    )
  ];

  static List<BoxShadow>? get text => kElevationToShadow[1];
}

class SunnyDimensions {
  static const double titleTextFont = 20;
  static const double listHeaderFont = 16;
  static const double listHeaderFontLarge = 18;
  static const double listHeaderFontSmall = 13;
  static const double standardPadding = 10;
  static const double halfPadding = 5;

  SunnyDimensions._();

  static const double largeBoxHeight = 150;
  static const double largeBoxWidth = 170;
}

class SunnySpacers {
  SunnySpacers._();

  static const vertical = SizedBox(height: SunnyDimensions.standardPadding);
}

class SunnyDeco {
  SunnyDeco._();

  static const borderSide = BorderSide(color: SunnyColors.disabledText);
  static const borderSideLight = BorderSide(color: SunnyColors.lightBorder);

  static const inputBorderInactive =
      BorderSide(color: SunnyColors.navBarInactive, width: 0.75);
  static const inputBorderNonFocused =
      BorderSide(color: SunnyColors.primaryAction, width: 0.5);
  static const inputBorderFocused =
      BorderSide(color: SunnyColors.primaryAction, width: 0.75);
}

const noPadding = EdgeInsets.zero;

class SunnyMargins {
  SunnyMargins._();

  static const EdgeInsets tlr = EdgeInsets.only(top: 10, left: 10, right: 10);
  static const EdgeInsets blr = EdgeInsets.only(bottom: 5, left: 10, right: 10);
  static EdgeInsets tapMargin = EdgeInsets.all(15);
  static const BorderRadius listItemRadius =
      BorderRadius.all(Radius.circular(8));
  static const EdgeInsets all = EdgeInsets.all(10);
  static const EdgeInsets half = EdgeInsets.all(5);
  static const EdgeInsets shim = half;

  /// Works around a weird bug where there's no padding from the top of the screen and an input
  static const EdgeInsets veryTop =
      EdgeInsets.only(left: 10, right: 10, top: 10.0);

  static EdgeInsets get dense => infoX.isIOS ? EdgeInsets.all(2) : zero;
  static const EdgeInsets emptyState = EdgeInsets.all(20);
  static const EdgeInsets sides = EdgeInsets.only(left: 10, right: 10);
  static const EdgeInsets horizScroller = EdgeInsets.only(left: 10, bottom: 10);
  static const EdgeInsets right = EdgeInsets.only(right: 10);
  static const EdgeInsets top = EdgeInsets.only(top: 10);
  static const EdgeInsets bottom = EdgeInsets.only(bottom: 10);
  static const EdgeInsets rightCurveWithBottom =
      EdgeInsets.only(right: 22, bottom: 1);
  static const EdgeInsets rightCurve = EdgeInsets.only(right: 22, bottom: 0);

  // Aligns with the edge of a box with a radius
  static const EdgeInsets leftCurve = EdgeInsets.only(left: 22, bottom: 0);
  static const EdgeInsets solidIconPadding = EdgeInsets.all(4);
  static const EdgeInsets listItemPadding =
      EdgeInsets.symmetric(vertical: 10.0, horizontal: 12.0);
  static const EdgeInsets listItemMargin =
      EdgeInsets.only(left: 10, right: 10, bottom: 10);
  static const EdgeInsets listHeaderMargin =
      EdgeInsets.only(left: 10, right: 10, top: 10, bottom: 10);
  static const EdgeInsets listHeaderMarginSmall =
      EdgeInsets.only(left: 10, right: 10, top: 10, bottom: 5);
  static const EdgeInsets listHeaderMarginBundle =
      EdgeInsets.only(left: 15, right: 10, top: 15, bottom: 12);
  static const EdgeInsets listHeaderMarginBundleHoriz =
      EdgeInsets.only(left: 10, right: 10, top: 5, bottom: 5);
  static const EdgeInsets horizontalListItemMargin =
      EdgeInsets.only(right: 10, bottom: 10);

  static const EdgeInsets textInputPadding =
      EdgeInsets.only(top: 15, bottom: 15);

  static const EdgeInsets left = EdgeInsets.only(left: 10);
  static const EdgeInsets horiz = EdgeInsets.only(left: 10, right: 10);
  static const EdgeInsets vert = EdgeInsets.only(top: 10, bottom: 10);
  static const EdgeInsets top_left = EdgeInsets.only(left: 10, top: 10);
  static const EdgeInsets bottom_right = EdgeInsets.only(bottom: 10, right: 10);
  static const EdgeInsets top_bottom = EdgeInsets.only(top: 10, bottom: 10);
  static const EdgeInsets avatarMargins = EdgeInsets.fromLTRB(0, 0, 10, 10);
  static const EdgeInsets stackedText = EdgeInsets.only(bottom: 3);
  static const EdgeInsets zero = EdgeInsets.zero;
}

class SunnyPickers {
  SunnyPickers._();

  static const backgroundColor = SunnyColors.white;
  static const bool useMagnifier = true;
  static const double itemExtent = 32;
  static const double magnification = 1.05;
  static const double offAxisFraction = 0.3;
}

class SunnyTextStyles {
  static const TextStyle heroButtonLabel = TextStyle(
      color: SunnyColors.white, fontSize: 18, fontWeight: FontWeight.w600);

  SunnyTextStyles._();

  static const listHeader = TextStyle(
    color: SunnyColors.indicatorText,
    fontSize: SunnyDimensions.listHeaderFont,
    fontWeight: FontWeight.w600,
  );

  static const segmentedControl = TextStyle(
    color: SunnyColors.primaryAction,
    fontSize: 13,
    fontWeight: FontWeight.w300,
  );

  static const selectedSegmentControl = TextStyle(
    color: SunnyColors.white,
    fontSize: 14,
    fontWeight: FontWeight.w400,
  );

  static const titleText = TextStyle(
    color: SunnyColors.indicatorText,
    fontSize: SunnyDimensions.titleTextFont,
    fontWeight: FontWeight.w400,
  );

  static const heroTitle = TextStyle(
//    fontFamily: ,
    color: SunnyColors.primaryText,
    fontSize: SunnyDimensions.titleTextFont,
    fontWeight: FontWeight.w900,
  );

  static const listHeaderSmall = TextStyle(
    color: SunnyColors.subTitleText,
    fontSize: SunnyDimensions.listHeaderFontSmall,
    fontWeight: FontWeight.w400,
  );

  static const userPackListHeader = TextStyle(
      color: SunnyColors.primaryText,
      fontSize: SunnyDimensions.listHeaderFont,
      fontWeight: FontWeight.w700,
      letterSpacing: 0.8);

  static const userPackListHeaderLight = TextStyle(
      color: SunnyColors.white,
      fontSize: SunnyDimensions.listHeaderFontSmall,
      fontWeight: FontWeight.w600,
      letterSpacing: 0.9);

  static TextStyle listHeaderSelected = listHeader.copyWith(
    color: SunnyColors.primaryText,
    fontWeight: FontWeight.w900,
  );

  /// Used on pack install page
  static TextStyle listSubheaderSelected = listHeader.copyWith(
    color: SunnyColors.primaryAction,
    fontWeight: FontWeight.w700,
    fontSize: SunnyDimensions.listHeaderFontSmall,
  );

  static TextStyle listHeaderSelectedUnderline =
      listHeaderSelected.copyWith(decoration: TextDecoration.underline);
  static const TextStyle appBarText =
      TextStyle(fontSize: 16, color: SunnyColors.inputControls);
  static const TextStyle appBarInput = TextStyle(
      fontSize: 16,
      fontWeight: FontWeight.normal,
      color: SunnyColors.inputControls);

  static const TextStyle textInput = secondary;

  static const TextStyle primaryTitle = TextStyle(
      fontSize: 20,
      fontWeight: FontWeight.w500,
      color: SunnyColors.primaryText);
  static const TextStyle largeHelpText = TextStyle(
      fontSize: 16,
      fontWeight: FontWeight.w500,
      color: SunnyColors.tertiaryText);
  static const TextStyle primary = TextStyle(
    fontSize: 13,
    fontWeight: FontWeight.w600,
    color: SunnyColors.primaryText,
  );

  static const TextStyle button = TextStyle(
    fontSize: 13,
    fontWeight: FontWeight.w700,
    color: SunnyColors.white,
  );

  static const TextStyle secondary = TextStyle(
      fontSize: 13,
      color: SunnyColors.secondaryText,
      fontWeight: FontWeight.w300);
  static const TextStyle timestamp = TextStyle(
    fontSize: 12,
    fontWeight: FontWeight.normal,
    color: SunnyColors.indicatorText,
  );
  static const TextStyle tertiary =
      TextStyle(fontSize: 11, color: SunnyColors.indicatorText);

  static const TextStyle finePrint = TextStyle(
    fontSize: 10,
    color: SunnyColors.primaryText,
    fontWeight: FontWeight.normal,
  );

  static const TextStyle notificationText = TextStyle(
      fontSize: 12, fontWeight: FontWeight.bold, color: SunnyColors.white);

//  static const TextStyle finePrint = TextStyle(
//    fontSize: 10,
//    color: SunnyColors.primaryText,
//    fontWeight: FontWeight.bold,
//  );
  static const TextStyle errorText = TextStyle(fontSize: 13, color: Colors.red);
  static const TextStyle secondaryActionText =
      TextStyle(fontSize: 14, color: SunnyColors.secondaryText);
  static const TextStyle actionItemPrimary =
      TextStyle(fontSize: 16, color: SunnyColors.secondaryText);
  static const TextStyle actionItemSecondary =
      TextStyle(fontSize: 14, color: SunnyColors.tertiaryText);
  static const TextStyle actionText = TextStyle(
    fontSize: 14,
    color: SunnyColors.primaryAction,
    fontWeight: FontWeight.w500,
  );
  static const TextStyle appBarLargeTitle = TextStyle(
    fontFamily: '.SF Pro Text',
    fontSize: 34.0,
    fontWeight: FontWeight.w800,
    letterSpacing: 0.36,
    color: CupertinoColors.black,
  );
  static TextStyle primaryActionText = TextStyle(
      fontSize: 16,
      fontWeight: FontWeight.w500,
      color: SunnyColors.primaryAction);
  static const TextStyle headerActionText =
      TextStyle(fontSize: 14, color: SunnyColors.primaryAction);

  static const TextStyle appBarSmallTitle =
      TextStyle(color: SunnyColors.primaryText);
  static const TextStyle groupAvatarTitle = TextStyle(
      fontSize: 14,
      fontWeight: FontWeight.w700,
      color: SunnyColors.primaryText);
  static const TextStyle bundleTitle = TextStyle(
      fontSize: 20,
      fontWeight: FontWeight.w700,
      letterSpacing: 1.1,
      color: SunnyColors.primaryText);

  // Form/input styles
  static const TextStyle pickerLabel =
      TextStyle(fontSize: 18, color: SunnyColors.indicatorText);
  static const TextStyle pickerText =
      TextStyle(fontSize: 20, color: SunnyColors.indicatorText);
  static const TextStyle emptyState =
      TextStyle(fontSize: 22, color: SunnyColors.indicatorText);
  static const TextStyle emptyStateDense =
      TextStyle(fontSize: 10, color: SunnyColors.disabledText);
  static const TextStyle stateValues = TextStyle(
      fontSize: 12,
      fontWeight: FontWeight.w600,
      color: SunnyColors.tertiaryText);

  static const TextStyle tabBarLabel = tertiary;

  static const TextStyle actionButtonCancel =
      TextStyle(color: SunnyColors.indicatorText);
}

class SunnyTextFieldStyles {
  SunnyTextFieldStyles._();

  /* https://material.io/design/components/text-fields.html#theming */
  static const TextStyle hintStyle =
      TextStyle(fontSize: 14, color: SunnyColors.primaryText, height: 1.6);
  static const TextStyle label =
      TextStyle(fontSize: 13, color: SunnyColors.indicatorText, height: 1.2);
  static const TextStyle inputText =
      TextStyle(fontSize: 14, color: SunnyColors.primaryText);
  static const TextStyle defaultText =
      TextStyle(fontSize: 14, color: SunnyColors.primaryText);

  /// See [SunnyThemes.defaultCupertinoTextField()]

  static final TextStyle inputPlaceholder = inputText.copyWith(
      color: SunnyColors.inputControls, fontWeight: FontWeight.normal);
}

class SunnyButtonTextStyle {
  SunnyButtonTextStyle._();

  static const TextStyle buttonText = TextStyle(
      fontSize: 16, fontWeight: FontWeight.w600, color: SunnyColors.white);
}
