import 'dart:async';

import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:spinner_button/spinner_button.dart';
import 'package:sunny_dart/sunny_dart.dart';
import 'package:sunny_essentials/sunny_essentials.dart';
import 'package:sunny_forms/sunny_colors.dart';

typedef FutureCallback<T> = FutureOr<T> Function(BuildContext context);
typedef ValueConsumer<T> = void Function(BuildContext context, T value);

class SunnyFormSubmit<T> extends StatefulWidget {
  final FutureCallback<T>? onPressed;
  final ValueConsumer<T>? onCompletion;
  final Widget label;
  final String? id;
  final Duration? minSubmitDuration;
//  final Color color;
  final bool? isLight;

  SunnyFormSubmit({
    Key? key,
    required this.onPressed,
    required this.onCompletion,
    this.isLight,
//    this.color,
    String? label,
    this.minSubmitDuration,
  })  : id = label,
        label = Text(label ?? "Save",
            style: isLight == true ? formSubmitLight : formSubmitDark),
        super(key: key);

  SunnyFormSubmit.ofWidget({
    Key? key,
    required this.onPressed,
    required this.onCompletion,
    this.isLight,
    required this.label,
    this.id,
    this.minSubmitDuration,
  }) : super(key: key);

  SunnyFormSubmit.popResult(
      {String? label,
      this.minSubmitDuration,
      this.isLight,
      Key? key,
      this.onPressed})
      : onCompletion = ((context, value) {
          if (value != null) {
            Navigator.pop(context, value);
          }
        }),
        label = Text(label ?? "Save",
            style: isLight == true ? formSubmitLight : formSubmitDark),
        id = label,
        super(key: key);

  @override
  _SunnyFormSubmitState createState() => _SunnyFormSubmitState<T>();
}

/// Button is light, text is dark
TextStyle get formSubmitLight => SunnyTextStyles.button.copyWith(
      color: sunnyColors.primaryColor,
    );

/// Button is dark, text is light
TextStyle get formSubmitDark => SunnyTextStyles.button;

class _SunnyFormSubmitState<T> extends State<SunnyFormSubmit<T?>> {
  bool submitting = false;

  @override
  Widget build(BuildContext context) {
    return Container(
      width: 200,
      height: 50,
      child: SpinnerButton(
        key: Key(widget.id ?? "sunny-form-submit"),
        elevation: 0,
        borderColor: sunnyColors.linkColor,
        borderRadius: BorderRadius.circular(8),
        borderWidth: 0.0,
        progressIndicatorColor:
            widget.isLight == true ? sunnyColors.primaryColor : Colors.white,
        child: widget.label,
        color:
            (widget.isLight == true ? Colors.white : sunnyColors.primaryColor),
        onPressed: (animation) {
          /// Prevent double submit
          if (submitting) return;

          setState(() {
            submitting = true;
            animation!.forward();
          });

          _submitResults(animation);
        },
      ),
    );
  }

  _submitResults(AnimationController? animation) async {
    try {
      await Future.wait<dynamic>([
        Future.sync(() => widget.onPressed!(context)).then((result) {
          widget.onCompletion?.call(context, result);
        }).catchError((err) {
          widget.onCompletion?.call(context, null);
        }),
        Future.delayed(widget.minSubmitDuration ?? 1.second),
      ]);
    } finally {
      if (this.mounted) {
        submitting = false;
        animation!.reverse().then((_) {
          if (mounted) {
            setState(() {});
          }
        }).ignore();
      }
    }
  }
}
