import 'package:collection_diff/list_diff_model.dart';
import 'package:sunny_data/sunny_data.dart';
import 'package:sunny_sdk_core/mverse.dart';
import 'package:sunny_dart/sunny_dart.dart';
import 'package:sunny_forms/sunny_list/record_key_map.dart';

class RecordStreamExtensions {
  const RecordStreamExtensions._();
}

extension ValueStreamIterableRecordExtensions<V extends Entity>
    on ValueStream<Iterable<Record<V>>> {
  ValueStream<Future<Iterable<V>>> resolveAll() {
    return this.mapEach((record) => record.future).awaitEach();
  }

  ValueStream<Iterable<V>> whereResolved() {
    return this.mapEach((record) => record.valueOrNull).whereNotNull()
        as ValueStream<Iterable<V>>;
  }
}

extension ValueStreamIterableMModelExtensions<V extends MModel>
    on ValueStream<Iterable<V>> {
  ValueStream<Map<String?, V>> toMap() {
    return this.map((models) => mapOf(models, keyOf: (model) => model.id));
  }
}

extension IterableMModelExtensions<V extends MModel> on Iterable<V> {
  Map<String?, V> byId() {
    return mapOf(this, keyOf: (model) => model.id);
  }

  Map<MKey?, V> toMapByKey() =>
      Map.fromEntries(this.map((v) => MapEntry(v.mkey, v)));

  Set<MKey?> get ids => map((v) => v.mkey).toSet();
}

extension ValueStreamOfListDiffs<T> on ValueStream<ListDiffs<T>> {
  /// Returns a stream containing the state of the underlying list of [T]s for each change reported
  ValueStream<List<T>> replacements() =>
      this.map((diffs) => diffs.args.replacement);
}

extension ValueStreamMapRecordExtensions<V> on ValueStream<Map<RecordKey, V>> {
  ValueStream<RecordKeyLookup<V>> toLookup() => this.map(recordKeyLookup);
}

extension MapRecordExtensions<V> on Map<RecordKey, V> {
  RecordKeyLookup<V> toLookup() => RecordKeyLookup(this);
}

extension StreamIterableRecord<V extends Entity>
    on Stream<Iterable<Record<V>>> {
  Stream<Future<Iterable<V>>> resolveAll() {
    final result = this
        .map((records) => Future.wait(records.map((record) => record.future)));
    return result;
  }

  Stream<Iterable<V>> whereResolved() {
    final result = this.map((records) => records
        .map((record) => record.valueOrNull)
        .whereNotNull() as Iterable<V>);
    return result;
  }
}

extension IterableRecordExtensions<V extends Entity> on Iterable<Record<V>> {
  Future<Iterable<V>> resolveAll() async =>
      await Future.wait(this.map((record) => record.future));

  Iterable<V> get resolved =>
      this.where((r) => r.isResolved == true).map((r) => r.value);
  List<V> get resolvedList =>
      this.where((r) => r.isResolved == true).map((r) => r.value).toList();
}
