import 'package:flutter/widgets.dart' hide Builder;

import 'list_model.dart';

/// Helps to create animated lists

typedef ListTileBuilder<T> = Widget Function(
    BuildContext context, T source, int? index, Animation<double>);

ListTileBuilder<T> defaultWidgetListTileBuilder<T>() =>
    (BuildContext context, T t, int? index, Animation<double> anim) {
      return t is Widget
          ? t
          : throw Exception("Can't render non-widget as a widget");
    };

typedef SwipeBuilder<T> = Widget Function(BuildContext context, Widget child,
    Animation<double> animation, int index, ListModel<T> list);

enum AnimationType { size, fade, slide }

Widget animated<T>(
  AnimationType animationType, {
  required String name,
  bool reversed = false,
  required Animation<double> animation,
  required Widget child,
}) {
  switch (animationType) {
    case AnimationType.size:
      return SizeTransition(
          key: Key("size-$name"),
          sizeFactor: animation,
          axis: Axis.vertical,
          axisAlignment: 0.0,
          child: child);
    case AnimationType.fade:
      return FadeTransition(
          key: Key("fade-$name"), opacity: animation, child: child);
    case AnimationType.slide:
      return SizeTransition(
        key: Key("size-$name"),
        sizeFactor: CurvedAnimation(
            curve: Interval(
              0.0,
              0.3,
              curve: Curves.decelerate,
            ),
            parent: animation),
        axis: Axis.vertical,
        axisAlignment: 0.0,
        child: SlideTransition(
          child: child,
          key: Key("size-slide-$name"),
          position: Tween<Offset>(
            begin: const Offset(-1, 0),
            end: Offset.zero,
          ).animate(animation),
        ),
      );

    default:
      return child;
  }
}

Widget slideIcon({Widget? child, EdgeInsets? padding}) => Container(
      padding: padding ?? EdgeInsets.only(right: 10, top: 6, bottom: 6),
      alignment: AlignmentDirectional.centerStart,
      child: child,
    );

Widget slideIconInner({Widget? child, EdgeInsets? padding}) => Container(
      padding: padding ?? EdgeInsets.only(right: 3, top: 6, bottom: 6),
      alignment: AlignmentDirectional.centerStart,
      child: child,
    );

typedef ListChangedListener<T> = void Function(Iterable<T> items);
