import 'package:flutter/material.dart';
import 'package:flutter/widgets.dart';
import 'package:flutter_platform_widgets/flutter_platform_widgets.dart';
import 'package:sunny_forms/fixed_dimension.dart';
import 'package:sunny_forms/sunny_colors.dart';

class SunnyListHeader extends StatelessWidget implements FixedHeight {
  final String text;
  final Widget? titleWidget;
  final Widget? trailing;
  final Color? backgroundColor;
  final Widget? leading;
  final EdgeInsets? margin;
  final bool? isSelected;
  final bool? useUnderlines;
  final TextStyle? style;
  final Decoration? decoration;

  @override
  double get maxHeight =>
      SunnyDimensions.listHeaderFont + SunnyDimensions.standardPadding;

  const SunnyListHeader(
    this.text, {
    Key? key,
    this.isSelected,
    this.titleWidget,
    this.useUnderlines,
    this.trailing,
    this.leading,
    this.style,
    this.margin,
    this.backgroundColor,
    this.decoration,
  }) : super(key: key);

  SunnyListHeader.primaryAction(
    this.text, {
    Key? key,
    this.isSelected,
    this.titleWidget,
    this.useUnderlines,
    this.trailing,
    this.leading,
    this.margin,
    this.backgroundColor,
    this.decoration,
  })  : style = SunnyTextStyles.listSubheaderSelected,
        super(key: key);

  @override
  Widget build(BuildContext context) {
    final selectedStyle = useUnderlines == true
        ? SunnyTextStyles.listHeaderSelectedUnderline
        : SunnyTextStyles.listHeaderSelected;
    final style = this.style ??
        (isSelected == true ? selectedStyle : SunnyTextStyles.listHeader);
    return Container(
      padding: this.margin ?? SunnyMargins.listHeaderMargin,
      color: backgroundColor,
      decoration: this.decoration,
      child: Row(
        mainAxisSize: MainAxisSize.max,
        mainAxisAlignment: MainAxisAlignment.spaceBetween,
        crossAxisAlignment: CrossAxisAlignment.center,
        children: [
          Wrap(
            crossAxisAlignment: WrapCrossAlignment.center,
            alignment: WrapAlignment.start,
            spacing: 5,
            children: [
              if (leading != null)
                Container(margin: SunnyMargins.zero, child: leading),
              PlatformText(text, style: style),
              if (titleWidget != null) titleWidget!,
            ],
          ),
          if (trailing != null) Container(child: trailing),
        ],
      ),
    );
  }
}
