import 'package:flutter/widgets.dart';
import 'package:flutter_mobx/flutter_mobx.dart';
import 'package:sunny_essentials/container.dart';
import 'package:sunny_forms/sunny_list/sunny_animated_list.dart';
import 'package:sunny_forms/sunny_list/sunny_list_base.dart';

class SunnySliverListState<T> extends SunnyListBaseState<T?> {
  GlobalKey<AnimatedListState>? globalKey;

  @override
  initState() {
    super.initState();
    globalKey = GlobalKey<AnimatedListState>();
    if (widget.header != null) {
      throw "SliverAnimatedListState doesn't support headers at this time";
    }
  }

  @override
  void insertItem(State? state, int? index, T? itemToInsert) {
    log.info("$debugName had ADD[$index]");
    globalKey!.currentState?.insertItem(index!);
  }

  @override
  void removeItem(State? state, int? index, T? itemToRemove) {
    log.info("$debugName had REMOVE[$index]");
    globalKey!.currentState?.removeItem(index!, (context, animation) {
      return animated(
        widget.dismissAnimationType,
        name: "remove-$index",
        reversed: true,
        animation: animation,
        child: itemBuilder(context, itemToRemove, index, animation),
      );
    });
  }

  @override
  void updateItem(State state, int index, T? oldItem, T? newItem) {
    log.info("$debugName had UPDATE[$index] (ignoring)");
  }

  @override
  void moveItem(State state, int fromIndex, int toIndex, T? itemToMove) {
    log.info("$debugName had MOVE[$fromIndex] to[$toIndex]");
  }

  @override
  Widget build(BuildContext context) {
    return Observer(
      builder: (_) {
        return isDisplayed
            ? SliverList(
                key: Key("sliver-list-${widget.id}"),
                delegate: SliverChildBuilderDelegate(
                  (context, index) => buildCompleteItem(
                      context, index, kAlwaysCompleteAnimation),
                  childCount: trackedList!.length,
                  addAutomaticKeepAlives: false,
                  findChildIndexCallback: (key) => keyToIndex[key],
                ),
              )
            : SliverToBoxAdapter(
                child: widget.emptyState?.call(context, this) ?? emptyBox);
      },
    );
  }
}
